def set_axis_bound(ax, left, right, lowwer, upper):
    x_left, x_right = ax.get_xlim()
    y_lower, y_upper = ax.get_ylim()
    ax.set_xbound(x_left + (x_right - x_left) * left,
                  x_left + (x_right - x_left) * right)
    ax.set_ybound(y_lower + (y_upper - y_lower) * lowwer,
                  y_lower + (y_upper - y_lower) * upper)


def robinson_bound(ax):
    ax.set_global()
    ax.axis('off')
    set_axis_bound(ax, 0.13, 0.96, 0.15, 0.99)


def lonlat_bound(ax):
    ax.set_global()
    ax.coastlines(color='#939698', linewidth=1)
    ax.set_xticks([-180, -120, -60, 0, 60, 120, 180])
    ax.set_xticklabels([])
    ax.set_yticks([-60, -30, 0, 30, 60, 90])
    ax.set_yticklabels([])
    ax.tick_params(axis="both", direction="in")
    ax.set_ylim(bottom=-60)



import numpy as np
import matplotlib as mpl


def rgb2hex(rgb):
    return ['#%02x%02x%02x' % tuple((np.array(i) * 1).astype(int)) for i in rgb]


def truncate_colormap(cmap, minval=0.0, maxval=1.0, n=256):
    cmap = mpl.colormaps[cmap] if isinstance(cmap, str) else cmap
    new_cmap = mpl.colors.LinearSegmentedColormap.from_list(
        f'trunc({cmap.name},{minval:.2f},{maxval:.2f})',
        cmap(np.linspace(minval, maxval, n)))
    return new_cmap


def boundary_cmap(colors, bounds):
    colors = [c if isinstance(c, str) else rgb2hex(c) for c in colors]
    bmap = mpl.colors.LinearSegmentedColormap.from_list(
        'boundary_cmap', list(zip(np.linspace(0, 1, len(colors)), colors)))
    norm = mpl.colors.BoundaryNorm(bounds, bmap.N)
    return bmap, norm



params = {
    'axes.labelsize': 12,
    'font.size': 12,
    'savefig.dpi': 1000,
    'axes.spines.right': False,
    'axes.spines.top': False,
    'legend.fontsize': 12,
    'xtick.labelsize': 12,
    'ytick.labelsize': 12,
    'text.usetex': False,
    'legend.labelspacing': 1,
    'legend.borderpad': 0.5,
    'legend.borderaxespad': 0.5,
    'axes.labelpad': 2,
    'axes.linewidth': 0.4,
    'patch.linewidth': 0.7,
    'lines.linewidth': 1,
    'lines.markersize': 8,
    # 'font.family': 'serif',
    'font.family': 'calibri',
    'xtick.major.pad': 1.5,
    'xtick.major.size': 2.5,
    'xtick.major.width': 0.4,
    'xtick.minor.pad': 1.2,
    'xtick.minor.size': 1.5,
    'xtick.minor.width': 0.3,
    'xtick.direction': 'in',
    'ytick.major.pad': 1.5,
    'ytick.major.size': 2.5,
    'ytick.major.width': 0.4,
    'ytick.minor.pad': 1.2,
    'ytick.minor.size': 1.5,
    'ytick.minor.width': 0.3,
    'ytick.direction': 'in',
    'font.serif': ["Times New Roman"]
}



#!/usr/bin/python
# -*-coding: utf-8 -*-
import numpy as np
from matplotlib.pyplot import cm
from matplotlib.figure import Figure
from matplotlib.patches import Rectangle, Patch
import matplotlib.font_manager as fm
from matplotlib.text import Text
from matplotlib.legend_handler import HandlerBase
import copy
import os
from itertools import product


def ceil(a, b):
    """
    Just like math.ceil
    """
    return int(a // b + bool(a % b))


def array_resize(array, length, array_len=None):
    """
    Resize array to given length. If the array is shorter than given length, repeat the array; If the array is longer
    than the length, trim the array.
    :param array: array
    :param length: target length
    :param array_len: if length of original array is known, pass it in here
    :return: axtended array
    """
    if not array_len:
        array_len = len(array)
    return array * (length // array_len) + array[:length % array_len]


_FONT_PATH = os.path.dirname(__file__)
FONT_FOLDER = {
    'brands': os.path.join(_FONT_PATH, 'icons', 'fa-brands-400.ttf'),
    'solid': os.path.join(_FONT_PATH, 'icons', 'fa-solid-900.ttf'),
    'regular': os.path.join(_FONT_PATH, 'icons', 'fa-regular-400.ttf')
}

class TextLegend(object):
    def __init__(self, text, color, **kwargs):
        self.text = text
        self.color = color
        self.kwargs = kwargs


class TextLegendHandler(HandlerBase):
    def __init__(self, font_file):
        super().__init__()
        self.font_file = FONT_FOLDER[font_file]

    def create_artists(self, legend, orig_handle, xdescent, ydescent, width, height, fontsize, trans):
        x = xdescent + width / 2.0
        y = ydescent + height / 2.0
        kwargs = {
            'horizontalalignment': 'center',
            'verticalalignment': 'center',
            'color': orig_handle.color,
            'fontproperties': fm.FontProperties(fname=self.font_file, size=fontsize)
        }
        kwargs.update(orig_handle.kwargs)
        annotation = Text(x, y, orig_handle.text, **kwargs)
        return [annotation]

    def legend_artist(self, legend, orig_handle, fontsize, handlebox):
        x0, y0 = handlebox.xdescent, handlebox.ydescent
        width, height = handlebox.width, handlebox.height
        kwargs = {
            'horizontalalignment': 'center',
            'verticalalignment': 'center',
            'color': orig_handle.color,
            'fontproperties': fm.FontProperties(fname=self.font_file, size=fontsize)
        }
        kwargs.update(orig_handle.kwargs)
        patch = Text(x=x0 + width / 2, y=height / 2 - y0,
                     text=orig_handle.text, **kwargs)
        handlebox.add_artist(patch)
        return patch


class Waffle(Figure):
    """

    A custom Figure class to make waffle charts.

    :param values: Numerical value of each category. If it is a dict, the keys would be used as labels.
    :type values: list|dict

    :param rows: The number of lines of the waffle chart. This is required if plots is not assigned.
    :type rows: int

    :param columns: The number of columns of the waffle chart.
        If it is not None, the total number of blocks would be decided through rows and columns. [Default None]
    :type columns: int

    :param colors: A list of colors for each category. Its length should be the same as values.
        Default values are from Set2 colormap.
    :type colors: list[str]|tuple[str]

    :param labels: The name of each category.
        If the values is a dict, this parameter would be replaced by the keys of values.
    :type labels: list[str]|tuple[str]

    :param legend: Parameters of matplotlib.pyplot.legend in a dict.
        E.g. {'loc': '', 'bbox_to_anchor': (,), ...}
        See full parameter list in https://matplotlib.org/api/_as_gen/matplotlib.pyplot.legend.html
    :type legend: dict

    :param icon_legend: Whether to use icon but not color bar in legend. [Default False]
    :type icon_legend: bool

    :param interval_ratio_x: Ratio of distance between blocks on X to block's width. [Default 0.2]
    :type interval_ratio_x: float

    :param interval_ratio_y: Ratio of distance between blocks on Y to block's height. [Default 0.2]
    :type interval_ratio_y: float

    :param block_aspect: The ratio of block's width to height. [Default 1]
    :type block_aspect: float

    :param cmap_name: Name of colormaps for default color, if colors is not assigned.
        See full list in https://matplotlib.org/examples/color/colormaps_reference.html [Default 'Set2']
    :type cmap_name: str

    :param title: Parameters of matplotlib.axes.Axes.set_title in a dict.
        E.g. {'label': '', 'fontdict': {}, 'loc': ''}
        See full parameter list in https://matplotlib.org/api/_as_gen/matplotlib.axes.Axes.set_title.html
    :type title: dict

    :param icons: Icon name of Font Awesome. If it is a string, all categories use the same icon;
        If it's a list or tuple of icons, the length should be the same as values.
        See the full list of Font Awesome on http://fontawesome.io/icons/ [Default None]
    :type icons: str|list[str]|tuple[str]

    :param icon_set: ('BRANDS', 'REGULAR', 'SOLID')
        The set of icons to be used. Visit https://fontawesome.com/cheatsheet to see which icons belong to which set.
        [Default 'SOLID']
    :type icon_set: str

    :param icon_size: Fint size of the icons. The default size is not fixed and depends on the block size.
    :type icon_size: int

    :param plot_anchor: {'C', 'SW', 'S', 'SE', 'E', 'NE', 'N', 'NW', 'W'}
        The alignment method of subplots.
        See details in https://matplotlib.org/devdocs/api/_as_gen/matplotlib.axes.Axes.set_anchor.html
        [Default 'W']
    :type plot_anchor: str

    :param plots: Location and parameters of Waffle class for subplots in a dict,
        with format like {loc: {subplot_args: values, }, }.
        loc is a 3-digit integer. If the three integers are I, J, and K,
        the subplot is the Ith plot on a grid with J rows and K columns.
        The parameters of subplots are the same as Waffle class parameters, excluding plots itself.
        Nested subplots is not supported.
        If any parameter of subplots is not assigned, it use the same parameter in Waffle class as default value.
    :type plots: dict
    :param plot_direction: {'NW', 'SW', 'NE', 'SE'}, the default value is SW.
    Change the starting location plotting the blocks
    'NW' means plots start at upper left and end at lower right.
    For 'SW', plots start at lower left and end at upper right.
    For 'NE', plots start at upper right and end at lower left.
    For 'SE', plots start at lower right and end at upper left.
    :type plot_direction: str
    """

    _direction_values = {
        'NW': {
            'column_order': 1,
            'row_order': -1,
        },
        'SW': {
            'column_order': 1,
            'row_order': 1,
        },
        'NE': {
            'column_order': -1,
            'row_order': 1,
        },
        'SE': {
            'column_order': -1,
            'row_order': -1,
        },
    }

    def __init__(self, *args, **kwargs):
        self.fig_args = {
            'values': kwargs.pop('values', []),
            'rows': kwargs.pop('rows', None),
            'columns': kwargs.pop('columns', None),
            'colors': kwargs.pop('colors', None),
            'labels': kwargs.pop('labels', None),
            'legend': kwargs.pop('legend', {}),
            'icon_legend': kwargs.pop('icon_legend', False),
            'interval_ratio_x': kwargs.pop('interval_ratio_x', 0.2),
            'interval_ratio_y': kwargs.pop('interval_ratio_y', 0.2),
            'block_aspect': kwargs.pop('block_aspect', 1),
            'cmap_name': kwargs.pop('cmap_name', 'Set2'),
            'title': kwargs.pop('title', None),
            'icons': kwargs.pop('icons', None),
            'icon_size': kwargs.pop('icon_size', None),
            'icon_set': kwargs.pop('icon_set', 'SOLID'),
            'plot_anchor': kwargs.pop('plot_anchor', 'W'),
            'plot_direction': kwargs.pop('plot_direction', 'SW'),
            'rotation': kwargs.pop('rotation', None),
            'show_num': kwargs.pop('show_num', False),
            'marker': kwargs.pop('icons', None),
            'marker_args': kwargs.pop('marker_args', {}),
            'rectangle_args': kwargs.pop('rectangle_args', {}),
            'nan': kwargs.pop('nan', None),
            'x_offset': kwargs.pop('x_offset', 0),
            'y_offset': kwargs.pop('y_offset', 0),
        }
        self.plots = kwargs.pop('plots', None)
        self.cover_plots = kwargs.pop('cover_plots', None)

        # # If plots is empty, make a single waffle chart
        # if self.plots is None:
        #     self.plots = {111: self.fig_args}

        Figure.__init__(self, *args, **kwargs)

        if self.plots is not None:
            for loc, setting in self.plots.items():
                self._waffle(loc, **copy.deepcopy(setting))
        if self.cover_plots is not None:
            for loc, setting in self.cover_plots:
                self._waffle(loc, **copy.deepcopy(setting))

        # Adjust the layout
        # self.set_tight_layout(True)

    def _waffle(self, loc, **kwargs):
        # _pa is the arguments for this single plot
        self._pa = kwargs

        # Append figure args to plot args
        plot_fig_args = copy.deepcopy(self.fig_args)
        for arg, v in plot_fig_args.items():
            if arg not in self._pa:
                self._pa[arg] = v

        element = np.delete(np.unique(self._pa['values']),
                            np.where(np.unique(self._pa['values']) == self._pa['nan'])[0])
        self.values_len = element.shape[0]

        # Build a color sequence if colors is empty
        if self._pa['colors'] is not None:
            if isinstance(self._pa['colors'], dict):
                self._pa['colors'] = {i: self._pa['colors'][i]
                                      for i in element}
            else:
                self._pa['colors'] = {i: self._pa['colors']
                                      for i in element}
        else:
            default_colors = cm.get_cmap(self._pa['cmap_name']).colors
            default_color_num = cm.get_cmap(self._pa['cmap_name']).N
            self._pa['colors'] = dict(zip(element,
                                          array_resize(array=default_colors, length=self.values_len, array_len=default_color_num)))

        rectangle_args = {}
        for k, v in self._pa['rectangle_args'].items():
            if isinstance(v, dict):
                rectangle_args[k] = {i: self._pa['rectangle_args'][k][i] for i in element}
            else:
                rectangle_args[k] = {i: self._pa['rectangle_args'][k] for i in element}
        if not any(i in rectangle_args.keys() for i in
                   ['color', 'edgecolor', 'facecolor', 'ec', 'fc']):
            rectangle_args['color'] = self._pa['colors']

        if isinstance(self._pa['values'], dict):
            if not self._pa['labels']:
                self._pa['labels'] = self._pa['values'].keys()
            self._pa['values'] = list(self._pa['values'].values())

        if self._pa['labels'] and len(self._pa['labels']) != self.values_len:
            raise ValueError("Length of labels doesn't match the values.")

        if self._pa['icons']:
            from .fontawesome_mapping import icons

            if self._pa['icon_set'] not in icons.keys():
                raise KeyError('icon_set should be one of {}'.format(
                    ', '.join(icons.keys())))

            # If icons is a string, convert it into a list of same icon. It's length is the label's length
            # '\uf26e' -> ['\uf26e', '\uf26e', '\uf26e', ]
            if isinstance(self._pa['icons'], str):
                self._pa['icons'] = {i: self._pa['icons']
                                     for i in element}

            if len(self._pa['icons']) < self.values_len:
                raise ValueError("Length of icons doesn't match the values.")

            self._pa['icons'] = {i: icons[self._pa['icon_set']]
                                 [self._pa['icons'][i]] for i in element}

        if self._pa['rotation']:
            self._pa['rotation'] = {i: self._pa['rotation'][i]
                                    for i in element}

        if isinstance(loc, tuple):
            self.ax = self.add_subplot(*loc, aspect='equal')
        else:
            self.ax = self.get_axes()[int(loc)]

        # Alignment of subplots
        self.ax.set_anchor(self._pa['plot_anchor'])

        # if column number is not given, use the values as number of blocks
        self._pa['rows'], self._pa['columns'] = self._pa['values'].shape[:2]

        # Absolute height of the plot
        figure_height = 1
        block_y_length = figure_height / (
            self._pa['rows'] + self._pa['rows'] *
            self._pa['interval_ratio_y'] - self._pa['interval_ratio_y']
        )
        block_x_length = self._pa['block_aspect'] * block_y_length

        # Define the limit of X, Y axis
        self.ax.axis(
            xmin=0,
            xmax=(
                self._pa['columns'] + self._pa['columns'] *
                self._pa['interval_ratio_x'] - self._pa['interval_ratio_x']
            ) * block_x_length,
            ymin=0,
            ymax=figure_height
        )

        # Default font size
        if self._pa['icons']:
            x, y = self.ax.transData.transform([(0, 0), (0, block_x_length)])
            prop = fm.FontProperties(
                fname=FONT_FOLDER[self._pa['icon_set']],
                size=self._pa['icon_size'] or int((y[1] - x[1]) / 16 * 12)
            )
        elif self._pa['show_num']:
            x, y = self.ax.transData.transform([(0, 0), (0, block_x_length)])
            prop = fm.FontProperties(
                family='consolas',
                size=self._pa['icon_size'] or int((y[1] - x[1]) / 16 * 12)
            )

        # Plot blocks
        x_full = (1 + self._pa['interval_ratio_x']) * block_x_length
        y_full = (1 + self._pa['interval_ratio_y']) * block_y_length

        plot_direction = self._pa['plot_direction'].upper()

        try:
            column_order = self._direction_values[plot_direction]['column_order']
            row_order = self._direction_values[plot_direction]['row_order']
        except KeyError:
            raise KeyError(
                "plot_direction should be one of 'NW', 'SW', 'NE', 'SE'")

        iter_cell = ((col, row) for col, row in product(range(self._pa['columns'])[::column_order], range(self._pa['rows'])[::row_order]) if
                     self._pa['values'][self._pa['rows'] - 1 - row, col] != self._pa['nan'])
        for col, row in iter_cell:
            x = x_full * col
            y = y_full * row

            if self._pa['icons']:
                rotation_arg = None
                if self._pa['rotation']:
                    rotation_arg = self._pa['rotation'][self._pa['values']
                                                        [self._pa['rows'] - 1 - row, col]]
                self.ax.text(
                    x=x + block_x_length * (1 / 2 + self._pa['x_offset']),
                    y=y + block_y_length * (1 / 2 + self._pa['y_offset']),
                    s=self._pa['icons'][self._pa['values']
                                        [self._pa['rows'] - 1 - row, col]],
                    color=self._pa['colors'][self._pa['values']
                                             [self._pa['rows'] - 1 - row, col]],
                    fontproperties=prop,
                    horizontalalignment='center',
                    verticalalignment='center',
                    rotation=rotation_arg,
                )
            elif self._pa['show_num']:
                self.ax.text(
                    x=x + block_x_length * (1 / 2 + self._pa['x_offset']),
                    y=y + block_y_length * (1 / 2 + self._pa['y_offset']),
                    s=str(self._pa['values']
                          [self._pa['rows'] - 1 - row, col]),
                    color=self._pa['colors'][self._pa['values']
                                             [self._pa['rows'] - 1 - row, col]],
                    fontproperties=prop,
                    horizontalalignment='center',
                    verticalalignment='center'
                )
            elif self._pa['marker']:
                self.ax.plot(
                    x + block_x_length * (1 / 2 + self._pa['x_offset']),
                    y + block_y_length * (1 / 2 + self._pa['y_offset']),
                    color=self._pa['colors'][self._pa['values']
                                             [self._pa['rows'] - 1 - row, col]],
                    marker=self._pa['marker'][self._pa['values']
                                              [self._pa['rows'] - 1 - row, col]],
                    **self._pa['marker_args']
                )
            else:
                temp_args = {k: v[self._pa['values'][self._pa['rows'] - 1 - row, col]]
                             for k, v in rectangle_args.items()}
                self.ax.add_artist(
                    Rectangle(xy=(x, y), width=block_x_length, clip_on=False,
                              height=block_y_length, **temp_args))

        # Add title
        if self._pa['title'] is not None:
            self.ax.set_title(**self._pa['title'])

        # Add legend
        if self._pa['labels'] or 'labels' in self._pa['legend']:
            if self._pa['icons'] and self._pa['icon_legend']:
                self._pa['legend']['handles'] = [
                    TextLegend(color=c, text=i) for c, i in zip(self._pa['colors'], self._pa['icons'])
                ]
                self._pa['legend']['handler_map'] = {
                    TextLegend: TextLegendHandler(self._pa['icon_set'])}
            # elif not self._pa['legend'].get('handles'):
            elif 'handles' not in self._pa['legend']:
                self._pa['legend']['handles'] = [
                    Patch(color=c, label=str(l)) for c, l in zip(self._pa['colors'], self._pa['labels'])
                ]

            # labels is an alias of legend['labels']
            if 'labels' not in self._pa['legend'] and self._pa['labels']:
                self._pa['legend']['labels'] = self._pa['labels']

            if 'handles' in self._pa['legend'] and 'labels' in self._pa['legend']:
                self.ax.legend(**self._pa['legend'])

        # Remove borders, ticks, etc.
        self.ax.axis('off')

    def remove(self):
        pass



import matplotlib.pyplot as plt
from geoplots._const import params


def init(figsize, widths=None, heights=None, wspace=0, hspace=0,
         left=0.005, right=0.995, bottom=0.005, top=0.995, **kwargs):
    plt.rcParams.update(params)
    f = plt.figure(figsize=figsize, **kwargs)
    ncols = 1 if widths is None else len(widths)
    nrows = 1 if heights is None else len(heights)
    gs = f.add_gridspec(ncols=ncols, nrows=nrows,
                        width_ratios=widths, wspace=wspace,
                        height_ratios=heights, hspace=hspace,
                        left=left, right=right, bottom=bottom, top=top)
    return f, gs


def tight(ax):
    ll, bb, ww, hh = ax.get_position().bounds
    bbox = ax.get_tightbbox()
    x, y = ax.figure.transFigure.inverted().transform([bbox.x0, bbox.y0])
    dx, dy = ll - x, bb - y
    ax.set_position([ll + dx, bb + dy, ww - dx, hh - dy])


def title(ax, label=None, **kwargs):
    label = chr(96 + len(ax.figure.axes)) if label is None else label
    label = chr(97 + label) if isinstance(label, int) else label
    kwargs['loc'] = kwargs.pop('loc', 'left')
    kwargs['y'] = kwargs.pop('y', 1)
    kwargs['pad'] = kwargs.pop('pad', 6)
    bbox = ax.get_tightbbox()
    x, _ = ax.transAxes.inverted().transform([bbox.x0, bbox.y0])
    kwargs['x'] = kwargs.pop('x', x)
    ax.set_title(label, **kwargs)



from geoplots._const import *
from geoplots.wrapper import *
from geoplots.cartopy import *
from geoplots.bound import *
from geoplots.legend import *
from geoplots.boxplot import *
from geoplots.heatmap import *
from geoplots.icongrid import *
from geoplots.color import *



import matplotlib as mpl


def stylize_boxplot(bp, colors):
    for i, box in enumerate(bp['boxes']):
        # set color for each box
        box.set_linewidth(0)
        boxCoords = list(zip(box.get_xdata(), box.get_ydata()))
        boxPolygon = mpl.patches.Polygon(
            boxCoords, facecolor=colors[i], linewidth=0)
        bp['whiskers'][0].axes.add_patch(boxPolygon)
        box.set_color(colors[i])

        # we have two whiskers!
        bp['whiskers'][i * 2].set_color(colors[i])
        bp['whiskers'][i * 2 + 1].set_color(colors[i])
        bp['whiskers'][i * 2].set_linewidth(1)
        bp['whiskers'][i * 2 + 1].set_linewidth(1)
        # top and bottom fliers
        if len(bp['fliers']) > 0:
            bp['fliers'][i].set(markerfacecolor=colors[i],
                                marker='o', alpha=0.75, markersize=2,
                                markeredgecolor='none')
        bp['medians'][i].set_color('black')
        bp['medians'][i].set_linewidth(1)
        # and 4 caps to remove
        xdata = bp['caps'][i * 2].get_xdata()
        dx = (xdata[1] - xdata[0]) / 4
        bp['caps'][i * 2].set(color=colors[i], linewidth=1,
                              xdata=bp['caps'][i * 2].get_xdata() + (dx, -dx))
        bp['caps'][i * 2 + 1].set(color=colors[i], linewidth=1,
                                  xdata=bp['caps'][i * 2 + 1].get_xdata() + (dx, -dx))



import itertools
import numpy as np
import matplotlib as mpl
from matplotlib.text import Text
from matplotlib.patches import Patch


# reverse order: flip(legend_elements, ncol), flip(labels, ncol)
def flip(items, ncol):
    return itertools.chain(*[items[i::ncol] for i in range(ncol)])


# Text legend: obj_0 = AnyObject("T", "#e41a1c")
class AnyObject(object):
    def __init__(self, text, color):
        self.text = text
        self.color = color


# 'handler_map': {obj_0: AnyObjectHandler()}
class AnyObjectHandler(object):
    def legend_artist(self, legend, orig_handle, fontsize, handlebox):
        x0, y0 = handlebox.xdescent, handlebox.ydescent
        width, height = handlebox.width, handlebox.height
        patch = Text(x=x0 + width / 2, y=height / 2 - y0,
                     text=orig_handle.text, color=orig_handle.color,
                     verticalalignment=u'center',
                     horizontalalignment=u'center',
                     multialignment=None, fontproperties=None,
                     linespacing=None, rotation_mode=None,
                     fontsize=20, family='consolas',)
        handlebox.add_artist(patch)
        return patch


# categorical legend: USA_Baseflow fig2_gages.py
def cat_legend(values, cmap, vmin=None, vmax=None, **style_kwds):
    values = np.asarray(values)
    categories = sorted(set(values))
    valuemap = dict((k, v) for (v, k) in enumerate(categories))
    values = np.array([valuemap[k] for k in values])
    mn = values[~np.isnan(values)].min() if vmin is None else vmin
    mx = values[~np.isnan(values)].max() if vmax is None else vmax

    norm = mpl.colors.Normalize(vmin=mn, vmax=mx)
    n_cmap = mpl.cm.ScalarMappable(norm=norm, cmap=cmap)

    patches = []
    for value, cat in enumerate(categories):
        patches.append(
            Patch(color=n_cmap.to_rgba(value), linewidth=1))
    return patches



# For Font Awesome version: 6.1.1

icons = {
    "solid": {
        "0": "0",
        "1": "1",
        "2": "2",
        "3": "3",
        "4": "4",
        "5": "5",
        "6": "6",
        "7": "7",
        "8": "8",
        "9": "9",
        "a": "A",
        "address-book": "\uf2b9",
        "contact-book": "\uf2b9",
        "address-card": "\uf2bb",
        "contact-card": "\uf2bb",
        "vcard": "\uf2bb",
        "align-center": "\uf037",
        "align-justify": "\uf039",
        "align-left": "\uf036",
        "align-right": "\uf038",
        "anchor": "\uf13d",
        "anchor-circle-check": "\ue4aa",
        "anchor-circle-exclamation": "\ue4ab",
        "anchor-circle-xmark": "\ue4ac",
        "anchor-lock": "\ue4ad",
        "angle-down": "\uf107",
        "angle-left": "\uf104",
        "angle-right": "\uf105",
        "angle-up": "\uf106",
        "angles-down": "\uf103",
        "angle-double-down": "\uf103",
        "angles-left": "\uf100",
        "angle-double-left": "\uf100",
        "angles-right": "\uf101",
        "angle-double-right": "\uf101",
        "angles-up": "\uf102",
        "angle-double-up": "\uf102",
        "ankh": "\uf644",
        "apple-whole": "\uf5d1",
        "apple-alt": "\uf5d1",
        "archway": "\uf557",
        "arrow-down": "\uf063",
        "arrow-down-1-9": "\uf162",
        "sort-numeric-asc": "\uf162",
        "sort-numeric-down": "\uf162",
        "arrow-down-9-1": "\uf886",
        "sort-numeric-desc": "\uf886",
        "sort-numeric-down-alt": "\uf886",
        "arrow-down-a-z": "\uf15d",
        "sort-alpha-asc": "\uf15d",
        "sort-alpha-down": "\uf15d",
        "arrow-down-long": "\uf175",
        "long-arrow-down": "\uf175",
        "arrow-down-short-wide": "\uf884",
        "sort-amount-desc": "\uf884",
        "sort-amount-down-alt": "\uf884",
        "arrow-down-up-across-line": "\ue4af",
        "arrow-down-up-lock": "\ue4b0",
        "arrow-down-wide-short": "\uf160",
        "sort-amount-asc": "\uf160",
        "sort-amount-down": "\uf160",
        "arrow-down-z-a": "\uf881",
        "sort-alpha-desc": "\uf881",
        "sort-alpha-down-alt": "\uf881",
        "arrow-left": "\uf060",
        "arrow-left-long": "\uf177",
        "long-arrow-left": "\uf177",
        "arrow-pointer": "\uf245",
        "mouse-pointer": "\uf245",
        "arrow-right": "\uf061",
        "arrow-right-arrow-left": "\uf0ec",
        "exchange": "\uf0ec",
        "arrow-right-from-bracket": "\uf08b",
        "sign-out": "\uf08b",
        "arrow-right-long": "\uf178",
        "long-arrow-right": "\uf178",
        "arrow-right-to-bracket": "\uf090",
        "sign-in": "\uf090",
        "arrow-right-to-city": "\ue4b3",
        "arrow-rotate-left": "\uf0e2",
        "arrow-left-rotate": "\uf0e2",
        "arrow-rotate-back": "\uf0e2",
        "arrow-rotate-backward": "\uf0e2",
        "undo": "\uf0e2",
        "arrow-rotate-right": "\uf01e",
        "arrow-right-rotate": "\uf01e",
        "arrow-rotate-forward": "\uf01e",
        "redo": "\uf01e",
        "arrow-trend-down": "\ue097",
        "arrow-trend-up": "\ue098",
        "arrow-turn-down": "\uf149",
        "level-down": "\uf149",
        "arrow-turn-up": "\uf148",
        "level-up": "\uf148",
        "arrow-up": "\uf062",
        "arrow-up-1-9": "\uf163",
        "sort-numeric-up": "\uf163",
        "arrow-up-9-1": "\uf887",
        "sort-numeric-up-alt": "\uf887",
        "arrow-up-a-z": "\uf15e",
        "sort-alpha-up": "\uf15e",
        "arrow-up-from-bracket": "\ue09a",
        "arrow-up-from-ground-water": "\ue4b5",
        "arrow-up-from-water-pump": "\ue4b6",
        "arrow-up-long": "\uf176",
        "long-arrow-up": "\uf176",
        "arrow-up-right-dots": "\ue4b7",
        "arrow-up-right-from-square": "\uf08e",
        "external-link": "\uf08e",
        "arrow-up-short-wide": "\uf885",
        "sort-amount-up-alt": "\uf885",
        "arrow-up-wide-short": "\uf161",
        "sort-amount-up": "\uf161",
        "arrow-up-z-a": "\uf882",
        "sort-alpha-up-alt": "\uf882",
        "arrows-down-to-line": "\ue4b8",
        "arrows-down-to-people": "\ue4b9",
        "arrows-left-right": "\uf07e",
        "arrows-h": "\uf07e",
        "arrows-left-right-to-line": "\ue4ba",
        "arrows-rotate": "\uf021",
        "refresh": "\uf021",
        "sync": "\uf021",
        "arrows-spin": "\ue4bb",
        "arrows-split-up-and-left": "\ue4bc",
        "arrows-to-circle": "\ue4bd",
        "arrows-to-dot": "\ue4be",
        "arrows-to-eye": "\ue4bf",
        "arrows-turn-right": "\ue4c0",
        "arrows-turn-to-dots": "\ue4c1",
        "arrows-up-down": "\uf07d",
        "arrows-v": "\uf07d",
        "arrows-up-down-left-right": "\uf047",
        "arrows": "\uf047",
        "arrows-up-to-line": "\ue4c2",
        "asterisk": "*",
        "at": "@",
        "atom": "\uf5d2",
        "audio-description": "\uf29e",
        "austral-sign": "\ue0a9",
        "award": "\uf559",
        "b": "B",
        "baby": "\uf77c",
        "baby-carriage": "\uf77d",
        "carriage-baby": "\uf77d",
        "backward": "\uf04a",
        "backward-fast": "\uf049",
        "fast-backward": "\uf049",
        "backward-step": "\uf048",
        "step-backward": "\uf048",
        "bacon": "\uf7e5",
        "bacteria": "\ue059",
        "bacterium": "\ue05a",
        "bag-shopping": "\uf290",
        "shopping-bag": "\uf290",
        "bahai": "\uf666",
        "baht-sign": "\ue0ac",
        "ban": "\uf05e",
        "cancel": "\uf05e",
        "ban-smoking": "\uf54d",
        "smoking-ban": "\uf54d",
        "bandage": "\uf462",
        "band-aid": "\uf462",
        "barcode": "\uf02a",
        "bars": "\uf0c9",
        "navicon": "\uf0c9",
        "bars-progress": "\uf828",
        "tasks-alt": "\uf828",
        "bars-staggered": "\uf550",
        "reorder": "\uf550",
        "stream": "\uf550",
        "baseball": "\uf433",
        "baseball-ball": "\uf433",
        "baseball-bat-ball": "\uf432",
        "basket-shopping": "\uf291",
        "shopping-basket": "\uf291",
        "basketball": "\uf434",
        "basketball-ball": "\uf434",
        "bath": "\uf2cd",
        "bathtub": "\uf2cd",
        "battery-empty": "\uf244",
        "battery-0": "\uf244",
        "battery-full": "\uf240",
        "battery": "\uf240",
        "battery-5": "\uf240",
        "battery-half": "\uf242",
        "battery-3": "\uf242",
        "battery-quarter": "\uf243",
        "battery-2": "\uf243",
        "battery-three-quarters": "\uf241",
        "battery-4": "\uf241",
        "bed": "\uf236",
        "bed-pulse": "\uf487",
        "procedures": "\uf487",
        "beer-mug-empty": "\uf0fc",
        "beer": "\uf0fc",
        "bell": "\uf0f3",
        "bell-concierge": "\uf562",
        "concierge-bell": "\uf562",
        "bell-slash": "\uf1f6",
        "bezier-curve": "\uf55b",
        "bicycle": "\uf206",
        "binoculars": "\uf1e5",
        "biohazard": "\uf780",
        "bitcoin-sign": "\ue0b4",
        "blender": "\uf517",
        "blender-phone": "\uf6b6",
        "blog": "\uf781",
        "bold": "\uf032",
        "bolt": "\uf0e7",
        "zap": "\uf0e7",
        "bolt-lightning": "\ue0b7",
        "bomb": "\uf1e2",
        "bone": "\uf5d7",
        "bong": "\uf55c",
        "book": "\uf02d",
        "book-atlas": "\uf558",
        "atlas": "\uf558",
        "book-bible": "\uf647",
        "bible": "\uf647",
        "book-bookmark": "\ue0bb",
        "book-journal-whills": "\uf66a",
        "journal-whills": "\uf66a",
        "book-medical": "\uf7e6",
        "book-open": "\uf518",
        "book-open-reader": "\uf5da",
        "book-reader": "\uf5da",
        "book-quran": "\uf687",
        "quran": "\uf687",
        "book-skull": "\uf6b7",
        "book-dead": "\uf6b7",
        "bookmark": "\uf02e",
        "border-all": "\uf84c",
        "border-none": "\uf850",
        "border-top-left": "\uf853",
        "border-style": "\uf853",
        "bore-hole": "\ue4c3",
        "bottle-droplet": "\ue4c4",
        "bottle-water": "\ue4c5",
        "bowl-food": "\ue4c6",
        "bowl-rice": "\ue2eb",
        "bowling-ball": "\uf436",
        "box": "\uf466",
        "box-archive": "\uf187",
        "archive": "\uf187",
        "box-open": "\uf49e",
        "box-tissue": "\ue05b",
        "boxes-packing": "\ue4c7",
        "boxes-stacked": "\uf468",
        "boxes": "\uf468",
        "boxes-alt": "\uf468",
        "braille": "\uf2a1",
        "brain": "\uf5dc",
        "brazilian-real-sign": "\ue46c",
        "bread-slice": "\uf7ec",
        "bridge": "\ue4c8",
        "bridge-circle-check": "\ue4c9",
        "bridge-circle-exclamation": "\ue4ca",
        "bridge-circle-xmark": "\ue4cb",
        "bridge-lock": "\ue4cc",
        "bridge-water": "\ue4ce",
        "briefcase": "\uf0b1",
        "briefcase-medical": "\uf469",
        "broom": "\uf51a",
        "broom-ball": "\uf458",
        "quidditch": "\uf458",
        "quidditch-broom-ball": "\uf458",
        "brush": "\uf55d",
        "bucket": "\ue4cf",
        "bug": "\uf188",
        "bug-slash": "\ue490",
        "bugs": "\ue4d0",
        "building": "\uf1ad",
        "building-circle-arrow-right": "\ue4d1",
        "building-circle-check": "\ue4d2",
        "building-circle-exclamation": "\ue4d3",
        "building-circle-xmark": "\ue4d4",
        "building-columns": "\uf19c",
        "bank": "\uf19c",
        "institution": "\uf19c",
        "museum": "\uf19c",
        "university": "\uf19c",
        "building-flag": "\ue4d5",
        "building-lock": "\ue4d6",
        "building-ngo": "\ue4d7",
        "building-shield": "\ue4d8",
        "building-un": "\ue4d9",
        "building-user": "\ue4da",
        "building-wheat": "\ue4db",
        "bullhorn": "\uf0a1",
        "bullseye": "\uf140",
        "burger": "\uf805",
        "hamburger": "\uf805",
        "burst": "\ue4dc",
        "bus": "\uf207",
        "bus-simple": "\uf55e",
        "bus-alt": "\uf55e",
        "business-time": "\uf64a",
        "briefcase-clock": "\uf64a",
        "c": "C",
        "cake-candles": "\uf1fd",
        "birthday-cake": "\uf1fd",
        "cake": "\uf1fd",
        "calculator": "\uf1ec",
        "calendar": "\uf133",
        "calendar-check": "\uf274",
        "calendar-day": "\uf783",
        "calendar-days": "\uf073",
        "calendar-alt": "\uf073",
        "calendar-minus": "\uf272",
        "calendar-plus": "\uf271",
        "calendar-week": "\uf784",
        "calendar-xmark": "\uf273",
        "calendar-times": "\uf273",
        "camera": "\uf030",
        "camera-alt": "\uf030",
        "camera-retro": "\uf083",
        "camera-rotate": "\ue0d8",
        "campground": "\uf6bb",
        "candy-cane": "\uf786",
        "cannabis": "\uf55f",
        "capsules": "\uf46b",
        "car": "\uf1b9",
        "automobile": "\uf1b9",
        "car-battery": "\uf5df",
        "battery-car": "\uf5df",
        "car-burst": "\uf5e1",
        "car-crash": "\uf5e1",
        "car-on": "\ue4dd",
        "car-rear": "\uf5de",
        "car-alt": "\uf5de",
        "car-side": "\uf5e4",
        "car-tunnel": "\ue4de",
        "caravan": "\uf8ff",
        "caret-down": "\uf0d7",
        "caret-left": "\uf0d9",
        "caret-right": "\uf0da",
        "caret-up": "\uf0d8",
        "carrot": "\uf787",
        "cart-arrow-down": "\uf218",
        "cart-flatbed": "\uf474",
        "dolly-flatbed": "\uf474",
        "cart-flatbed-suitcase": "\uf59d",
        "luggage-cart": "\uf59d",
        "cart-plus": "\uf217",
        "cart-shopping": "\uf07a",
        "shopping-cart": "\uf07a",
        "cash-register": "\uf788",
        "cat": "\uf6be",
        "cedi-sign": "\ue0df",
        "cent-sign": "\ue3f5",
        "certificate": "\uf0a3",
        "chair": "\uf6c0",
        "chalkboard": "\uf51b",
        "blackboard": "\uf51b",
        "chalkboard-user": "\uf51c",
        "chalkboard-teacher": "\uf51c",
        "champagne-glasses": "\uf79f",
        "glass-cheers": "\uf79f",
        "charging-station": "\uf5e7",
        "chart-area": "\uf1fe",
        "area-chart": "\uf1fe",
        "chart-bar": "\uf080",
        "bar-chart": "\uf080",
        "chart-column": "\ue0e3",
        "chart-gantt": "\ue0e4",
        "chart-line": "\uf201",
        "line-chart": "\uf201",
        "chart-pie": "\uf200",
        "pie-chart": "\uf200",
        "chart-simple": "\ue473",
        "check": "\uf00c",
        "check-double": "\uf560",
        "check-to-slot": "\uf772",
        "vote-yea": "\uf772",
        "cheese": "\uf7ef",
        "chess": "\uf439",
        "chess-bishop": "\uf43a",
        "chess-board": "\uf43c",
        "chess-king": "\uf43f",
        "chess-knight": "\uf441",
        "chess-pawn": "\uf443",
        "chess-queen": "\uf445",
        "chess-rook": "\uf447",
        "chevron-down": "\uf078",
        "chevron-left": "\uf053",
        "chevron-right": "\uf054",
        "chevron-up": "\uf077",
        "child": "\uf1ae",
        "child-dress": "\ue59c",
        "child-reaching": "\ue59d",
        "child-rifle": "\ue4e0",
        "children": "\ue4e1",
        "church": "\uf51d",
        "circle": "\uf111",
        "circle-arrow-down": "\uf0ab",
        "arrow-circle-down": "\uf0ab",
        "circle-arrow-left": "\uf0a8",
        "arrow-circle-left": "\uf0a8",
        "circle-arrow-right": "\uf0a9",
        "arrow-circle-right": "\uf0a9",
        "circle-arrow-up": "\uf0aa",
        "arrow-circle-up": "\uf0aa",
        "circle-check": "\uf058",
        "check-circle": "\uf058",
        "circle-chevron-down": "\uf13a",
        "chevron-circle-down": "\uf13a",
        "circle-chevron-left": "\uf137",
        "chevron-circle-left": "\uf137",
        "circle-chevron-right": "\uf138",
        "chevron-circle-right": "\uf138",
        "circle-chevron-up": "\uf139",
        "chevron-circle-up": "\uf139",
        "circle-dollar-to-slot": "\uf4b9",
        "donate": "\uf4b9",
        "circle-dot": "\uf192",
        "dot-circle": "\uf192",
        "circle-down": "\uf358",
        "arrow-alt-circle-down": "\uf358",
        "circle-exclamation": "\uf06a",
        "exclamation-circle": "\uf06a",
        "circle-h": "\uf47e",
        "hospital-symbol": "\uf47e",
        "circle-half-stroke": "\uf042",
        "adjust": "\uf042",
        "circle-info": "\uf05a",
        "info-circle": "\uf05a",
        "circle-left": "\uf359",
        "arrow-alt-circle-left": "\uf359",
        "circle-minus": "\uf056",
        "minus-circle": "\uf056",
        "circle-nodes": "\ue4e2",
        "circle-notch": "\uf1ce",
        "circle-pause": "\uf28b",
        "pause-circle": "\uf28b",
        "circle-play": "\uf144",
        "play-circle": "\uf144",
        "circle-plus": "\uf055",
        "plus-circle": "\uf055",
        "circle-question": "\uf059",
        "question-circle": "\uf059",
        "circle-radiation": "\uf7ba",
        "radiation-alt": "\uf7ba",
        "circle-right": "\uf35a",
        "arrow-alt-circle-right": "\uf35a",
        "circle-stop": "\uf28d",
        "stop-circle": "\uf28d",
        "circle-up": "\uf35b",
        "arrow-alt-circle-up": "\uf35b",
        "circle-user": "\uf2bd",
        "user-circle": "\uf2bd",
        "circle-xmark": "\uf057",
        "times-circle": "\uf057",
        "xmark-circle": "\uf057",
        "city": "\uf64f",
        "clapperboard": "\ue131",
        "clipboard": "\uf328",
        "clipboard-check": "\uf46c",
        "clipboard-list": "\uf46d",
        "clipboard-question": "\ue4e3",
        "clipboard-user": "\uf7f3",
        "clock": "\uf017",
        "clock-four": "\uf017",
        "clock-rotate-left": "\uf1da",
        "history": "\uf1da",
        "clone": "\uf24d",
        "closed-captioning": "\uf20a",
        "cloud": "\uf0c2",
        "cloud-arrow-down": "\uf0ed",
        "cloud-download": "\uf0ed",
        "cloud-download-alt": "\uf0ed",
        "cloud-arrow-up": "\uf0ee",
        "cloud-upload": "\uf0ee",
        "cloud-upload-alt": "\uf0ee",
        "cloud-bolt": "\uf76c",
        "thunderstorm": "\uf76c",
        "cloud-meatball": "\uf73b",
        "cloud-moon": "\uf6c3",
        "cloud-moon-rain": "\uf73c",
        "cloud-rain": "\uf73d",
        "cloud-showers-heavy": "\uf740",
        "cloud-showers-water": "\ue4e4",
        "cloud-sun": "\uf6c4",
        "cloud-sun-rain": "\uf743",
        "clover": "\ue139",
        "code": "\uf121",
        "code-branch": "\uf126",
        "code-commit": "\uf386",
        "code-compare": "\ue13a",
        "code-fork": "\ue13b",
        "code-merge": "\uf387",
        "code-pull-request": "\ue13c",
        "coins": "\uf51e",
        "colon-sign": "\ue140",
        "comment": "\uf075",
        "comment-dollar": "\uf651",
        "comment-dots": "\uf4ad",
        "commenting": "\uf4ad",
        "comment-medical": "\uf7f5",
        "comment-slash": "\uf4b3",
        "comment-sms": "\uf7cd",
        "sms": "\uf7cd",
        "comments": "\uf086",
        "comments-dollar": "\uf653",
        "compact-disc": "\uf51f",
        "compass": "\uf14e",
        "compass-drafting": "\uf568",
        "drafting-compass": "\uf568",
        "compress": "\uf066",
        "computer": "\ue4e5",
        "computer-mouse": "\uf8cc",
        "mouse": "\uf8cc",
        "cookie": "\uf563",
        "cookie-bite": "\uf564",
        "copy": "\uf0c5",
        "copyright": "\uf1f9",
        "couch": "\uf4b8",
        "cow": "\uf6c8",
        "credit-card": "\uf09d",
        "credit-card-alt": "\uf09d",
        "crop": "\uf125",
        "crop-simple": "\uf565",
        "crop-alt": "\uf565",
        "cross": "\uf654",
        "crosshairs": "\uf05b",
        "crow": "\uf520",
        "crown": "\uf521",
        "crutch": "\uf7f7",
        "cruzeiro-sign": "\ue152",
        "cube": "\uf1b2",
        "cubes": "\uf1b3",
        "cubes-stacked": "\ue4e6",
        "d": "D",
        "database": "\uf1c0",
        "delete-left": "\uf55a",
        "backspace": "\uf55a",
        "democrat": "\uf747",
        "desktop": "\uf390",
        "desktop-alt": "\uf390",
        "dharmachakra": "\uf655",
        "diagram-next": "\ue476",
        "diagram-predecessor": "\ue477",
        "diagram-project": "\uf542",
        "project-diagram": "\uf542",
        "diagram-successor": "\ue47a",
        "diamond": "\uf219",
        "diamond-turn-right": "\uf5eb",
        "directions": "\uf5eb",
        "dice": "\uf522",
        "dice-d20": "\uf6cf",
        "dice-d6": "\uf6d1",
        "dice-five": "\uf523",
        "dice-four": "\uf524",
        "dice-one": "\uf525",
        "dice-six": "\uf526",
        "dice-three": "\uf527",
        "dice-two": "\uf528",
        "disease": "\uf7fa",
        "display": "\ue163",
        "divide": "\uf529",
        "dna": "\uf471",
        "dog": "\uf6d3",
        "dollar-sign": "$",
        "dollar": "$",
        "usd": "$",
        "dolly": "\uf472",
        "dolly-box": "\uf472",
        "dong-sign": "\ue169",
        "door-closed": "\uf52a",
        "door-open": "\uf52b",
        "dove": "\uf4ba",
        "down-left-and-up-right-to-center": "\uf422",
        "compress-alt": "\uf422",
        "down-long": "\uf309",
        "long-arrow-alt-down": "\uf309",
        "download": "\uf019",
        "dragon": "\uf6d5",
        "draw-polygon": "\uf5ee",
        "droplet": "\uf043",
        "tint": "\uf043",
        "droplet-slash": "\uf5c7",
        "tint-slash": "\uf5c7",
        "drum": "\uf569",
        "drum-steelpan": "\uf56a",
        "drumstick-bite": "\uf6d7",
        "dumbbell": "\uf44b",
        "dumpster": "\uf793",
        "dumpster-fire": "\uf794",
        "dungeon": "\uf6d9",
        "e": "E",
        "ear-deaf": "\uf2a4",
        "deaf": "\uf2a4",
        "deafness": "\uf2a4",
        "hard-of-hearing": "\uf2a4",
        "ear-listen": "\uf2a2",
        "assistive-listening-systems": "\uf2a2",
        "earth-africa": "\uf57c",
        "globe-africa": "\uf57c",
        "earth-americas": "\uf57d",
        "earth": "\uf57d",
        "earth-america": "\uf57d",
        "globe-americas": "\uf57d",
        "earth-asia": "\uf57e",
        "globe-asia": "\uf57e",
        "earth-europe": "\uf7a2",
        "globe-europe": "\uf7a2",
        "earth-oceania": "\ue47b",
        "globe-oceania": "\ue47b",
        "egg": "\uf7fb",
        "eject": "\uf052",
        "elevator": "\ue16d",
        "ellipsis": "\uf141",
        "ellipsis-h": "\uf141",
        "ellipsis-vertical": "\uf142",
        "ellipsis-v": "\uf142",
        "envelope": "\uf0e0",
        "envelope-circle-check": "\ue4e8",
        "envelope-open": "\uf2b6",
        "envelope-open-text": "\uf658",
        "envelopes-bulk": "\uf674",
        "mail-bulk": "\uf674",
        "equals": "=",
        "eraser": "\uf12d",
        "ethernet": "\uf796",
        "euro-sign": "\uf153",
        "eur": "\uf153",
        "euro": "\uf153",
        "exclamation": "!",
        "expand": "\uf065",
        "explosion": "\ue4e9",
        "eye": "\uf06e",
        "eye-dropper": "\uf1fb",
        "eye-dropper-empty": "\uf1fb",
        "eyedropper": "\uf1fb",
        "eye-low-vision": "\uf2a8",
        "low-vision": "\uf2a8",
        "eye-slash": "\uf070",
        "f": "F",
        "face-angry": "\uf556",
        "angry": "\uf556",
        "face-dizzy": "\uf567",
        "dizzy": "\uf567",
        "face-flushed": "\uf579",
        "flushed": "\uf579",
        "face-frown": "\uf119",
        "frown": "\uf119",
        "face-frown-open": "\uf57a",
        "frown-open": "\uf57a",
        "face-grimace": "\uf57f",
        "grimace": "\uf57f",
        "face-grin": "\uf580",
        "grin": "\uf580",
        "face-grin-beam": "\uf582",
        "grin-beam": "\uf582",
        "face-grin-beam-sweat": "\uf583",
        "grin-beam-sweat": "\uf583",
        "face-grin-hearts": "\uf584",
        "grin-hearts": "\uf584",
        "face-grin-squint": "\uf585",
        "grin-squint": "\uf585",
        "face-grin-squint-tears": "\uf586",
        "grin-squint-tears": "\uf586",
        "face-grin-stars": "\uf587",
        "grin-stars": "\uf587",
        "face-grin-tears": "\uf588",
        "grin-tears": "\uf588",
        "face-grin-tongue": "\uf589",
        "grin-tongue": "\uf589",
        "face-grin-tongue-squint": "\uf58a",
        "grin-tongue-squint": "\uf58a",
        "face-grin-tongue-wink": "\uf58b",
        "grin-tongue-wink": "\uf58b",
        "face-grin-wide": "\uf581",
        "grin-alt": "\uf581",
        "face-grin-wink": "\uf58c",
        "grin-wink": "\uf58c",
        "face-kiss": "\uf596",
        "kiss": "\uf596",
        "face-kiss-beam": "\uf597",
        "kiss-beam": "\uf597",
        "face-kiss-wink-heart": "\uf598",
        "kiss-wink-heart": "\uf598",
        "face-laugh": "\uf599",
        "laugh": "\uf599",
        "face-laugh-beam": "\uf59a",
        "laugh-beam": "\uf59a",
        "face-laugh-squint": "\uf59b",
        "laugh-squint": "\uf59b",
        "face-laugh-wink": "\uf59c",
        "laugh-wink": "\uf59c",
        "face-meh": "\uf11a",
        "meh": "\uf11a",
        "face-meh-blank": "\uf5a4",
        "meh-blank": "\uf5a4",
        "face-rolling-eyes": "\uf5a5",
        "meh-rolling-eyes": "\uf5a5",
        "face-sad-cry": "\uf5b3",
        "sad-cry": "\uf5b3",
        "face-sad-tear": "\uf5b4",
        "sad-tear": "\uf5b4",
        "face-smile": "\uf118",
        "smile": "\uf118",
        "face-smile-beam": "\uf5b8",
        "smile-beam": "\uf5b8",
        "face-smile-wink": "\uf4da",
        "smile-wink": "\uf4da",
        "face-surprise": "\uf5c2",
        "surprise": "\uf5c2",
        "face-tired": "\uf5c8",
        "tired": "\uf5c8",
        "fan": "\uf863",
        "faucet": "\ue005",
        "faucet-drip": "\ue006",
        "fax": "\uf1ac",
        "feather": "\uf52d",
        "feather-pointed": "\uf56b",
        "feather-alt": "\uf56b",
        "ferry": "\ue4ea",
        "file": "\uf15b",
        "file-arrow-down": "\uf56d",
        "file-download": "\uf56d",
        "file-arrow-up": "\uf574",
        "file-upload": "\uf574",
        "file-audio": "\uf1c7",
        "file-circle-check": "\ue493",
        "file-circle-exclamation": "\ue4eb",
        "file-circle-minus": "\ue4ed",
        "file-circle-plus": "\ue4ee",
        "file-circle-question": "\ue4ef",
        "file-circle-xmark": "\ue494",
        "file-code": "\uf1c9",
        "file-contract": "\uf56c",
        "file-csv": "\uf6dd",
        "file-excel": "\uf1c3",
        "file-export": "\uf56e",
        "arrow-right-from-file": "\uf56e",
        "file-image": "\uf1c5",
        "file-import": "\uf56f",
        "arrow-right-to-file": "\uf56f",
        "file-invoice": "\uf570",
        "file-invoice-dollar": "\uf571",
        "file-lines": "\uf15c",
        "file-alt": "\uf15c",
        "file-text": "\uf15c",
        "file-medical": "\uf477",
        "file-pdf": "\uf1c1",
        "file-pen": "\uf31c",
        "file-edit": "\uf31c",
        "file-powerpoint": "\uf1c4",
        "file-prescription": "\uf572",
        "file-shield": "\ue4f0",
        "file-signature": "\uf573",
        "file-video": "\uf1c8",
        "file-waveform": "\uf478",
        "file-medical-alt": "\uf478",
        "file-word": "\uf1c2",
        "file-zipper": "\uf1c6",
        "file-archive": "\uf1c6",
        "fill": "\uf575",
        "fill-drip": "\uf576",
        "film": "\uf008",
        "filter": "\uf0b0",
        "filter-circle-dollar": "\uf662",
        "funnel-dollar": "\uf662",
        "filter-circle-xmark": "\ue17b",
        "fingerprint": "\uf577",
        "fire": "\uf06d",
        "fire-burner": "\ue4f1",
        "fire-extinguisher": "\uf134",
        "fire-flame-curved": "\uf7e4",
        "fire-alt": "\uf7e4",
        "fire-flame-simple": "\uf46a",
        "burn": "\uf46a",
        "fish": "\uf578",
        "fish-fins": "\ue4f2",
        "flag": "\uf024",
        "flag-checkered": "\uf11e",
        "flag-usa": "\uf74d",
        "flask": "\uf0c3",
        "flask-vial": "\ue4f3",
        "floppy-disk": "\uf0c7",
        "save": "\uf0c7",
        "florin-sign": "\ue184",
        "folder": "\uf07b",
        "folder-blank": "\uf07b",
        "folder-closed": "\ue185",
        "folder-minus": "\uf65d",
        "folder-open": "\uf07c",
        "folder-plus": "\uf65e",
        "folder-tree": "\uf802",
        "font": "\uf031",
        "font-awesome": "\uf2b4",
        "font-awesome-flag": "\uf2b4",
        "font-awesome-logo-full": "\uf2b4",
        "football": "\uf44e",
        "football-ball": "\uf44e",
        "forward": "\uf04e",
        "forward-fast": "\uf050",
        "fast-forward": "\uf050",
        "forward-step": "\uf051",
        "step-forward": "\uf051",
        "franc-sign": "\ue18f",
        "frog": "\uf52e",
        "futbol": "\uf1e3",
        "futbol-ball": "\uf1e3",
        "soccer-ball": "\uf1e3",
        "g": "G",
        "gamepad": "\uf11b",
        "gas-pump": "\uf52f",
        "gauge": "\uf624",
        "dashboard": "\uf624",
        "gauge-med": "\uf624",
        "tachometer-alt-average": "\uf624",
        "gauge-high": "\uf625",
        "tachometer-alt": "\uf625",
        "tachometer-alt-fast": "\uf625",
        "gauge-simple": "\uf629",
        "gauge-simple-med": "\uf629",
        "tachometer-average": "\uf629",
        "gauge-simple-high": "\uf62a",
        "tachometer": "\uf62a",
        "tachometer-fast": "\uf62a",
        "gavel": "\uf0e3",
        "legal": "\uf0e3",
        "gear": "\uf013",
        "cog": "\uf013",
        "gears": "\uf085",
        "cogs": "\uf085",
        "gem": "\uf3a5",
        "genderless": "\uf22d",
        "ghost": "\uf6e2",
        "gift": "\uf06b",
        "gifts": "\uf79c",
        "glass-water": "\ue4f4",
        "glass-water-droplet": "\ue4f5",
        "glasses": "\uf530",
        "globe": "\uf0ac",
        "golf-ball-tee": "\uf450",
        "golf-ball": "\uf450",
        "gopuram": "\uf664",
        "graduation-cap": "\uf19d",
        "mortar-board": "\uf19d",
        "greater-than": ">",
        "greater-than-equal": "\uf532",
        "grip": "\uf58d",
        "grip-horizontal": "\uf58d",
        "grip-lines": "\uf7a4",
        "grip-lines-vertical": "\uf7a5",
        "grip-vertical": "\uf58e",
        "group-arrows-rotate": "\ue4f6",
        "guarani-sign": "\ue19a",
        "guitar": "\uf7a6",
        "gun": "\ue19b",
        "h": "H",
        "hammer": "\uf6e3",
        "hamsa": "\uf665",
        "hand": "\uf256",
        "hand-paper": "\uf256",
        "hand-back-fist": "\uf255",
        "hand-rock": "\uf255",
        "hand-dots": "\uf461",
        "allergies": "\uf461",
        "hand-fist": "\uf6de",
        "fist-raised": "\uf6de",
        "hand-holding": "\uf4bd",
        "hand-holding-dollar": "\uf4c0",
        "hand-holding-usd": "\uf4c0",
        "hand-holding-droplet": "\uf4c1",
        "hand-holding-water": "\uf4c1",
        "hand-holding-hand": "\ue4f7",
        "hand-holding-heart": "\uf4be",
        "hand-holding-medical": "\ue05c",
        "hand-lizard": "\uf258",
        "hand-middle-finger": "\uf806",
        "hand-peace": "\uf25b",
        "hand-point-down": "\uf0a7",
        "hand-point-left": "\uf0a5",
        "hand-point-right": "\uf0a4",
        "hand-point-up": "\uf0a6",
        "hand-pointer": "\uf25a",
        "hand-scissors": "\uf257",
        "hand-sparkles": "\ue05d",
        "hand-spock": "\uf259",
        "handcuffs": "\ue4f8",
        "hands": "\uf2a7",
        "sign-language": "\uf2a7",
        "signing": "\uf2a7",
        "hands-asl-interpreting": "\uf2a3",
        "american-sign-language-interpreting": "\uf2a3",
        "asl-interpreting": "\uf2a3",
        "hands-american-sign-language-interpreting": "\uf2a3",
        "hands-bound": "\ue4f9",
        "hands-bubbles": "\ue05e",
        "hands-wash": "\ue05e",
        "hands-clapping": "\ue1a8",
        "hands-holding": "\uf4c2",
        "hands-holding-child": "\ue4fa",
        "hands-holding-circle": "\ue4fb",
        "hands-praying": "\uf684",
        "praying-hands": "\uf684",
        "handshake": "\uf2b5",
        "handshake-angle": "\uf4c4",
        "hands-helping": "\uf4c4",
        "handshake-simple": "\uf4c6",
        "handshake-alt": "\uf4c6",
        "handshake-simple-slash": "\ue05f",
        "handshake-alt-slash": "\ue05f",
        "handshake-slash": "\ue060",
        "hanukiah": "\uf6e6",
        "hard-drive": "\uf0a0",
        "hdd": "\uf0a0",
        "hashtag": "#",
        "hat-cowboy": "\uf8c0",
        "hat-cowboy-side": "\uf8c1",
        "hat-wizard": "\uf6e8",
        "head-side-cough": "\ue061",
        "head-side-cough-slash": "\ue062",
        "head-side-mask": "\ue063",
        "head-side-virus": "\ue064",
        "heading": "\uf1dc",
        "header": "\uf1dc",
        "headphones": "\uf025",
        "headphones-simple": "\uf58f",
        "headphones-alt": "\uf58f",
        "headset": "\uf590",
        "heart": "\uf004",
        "heart-circle-bolt": "\ue4fc",
        "heart-circle-check": "\ue4fd",
        "heart-circle-exclamation": "\ue4fe",
        "heart-circle-minus": "\ue4ff",
        "heart-circle-plus": "\ue500",
        "heart-circle-xmark": "\ue501",
        "heart-crack": "\uf7a9",
        "heart-broken": "\uf7a9",
        "heart-pulse": "\uf21e",
        "heartbeat": "\uf21e",
        "helicopter": "\uf533",
        "helicopter-symbol": "\ue502",
        "helmet-safety": "\uf807",
        "hard-hat": "\uf807",
        "hat-hard": "\uf807",
        "helmet-un": "\ue503",
        "highlighter": "\uf591",
        "hill-avalanche": "\ue507",
        "hill-rockslide": "\ue508",
        "hippo": "\uf6ed",
        "hockey-puck": "\uf453",
        "holly-berry": "\uf7aa",
        "horse": "\uf6f0",
        "horse-head": "\uf7ab",
        "hospital": "\uf0f8",
        "hospital-alt": "\uf0f8",
        "hospital-wide": "\uf0f8",
        "hospital-user": "\uf80d",
        "hot-tub-person": "\uf593",
        "hot-tub": "\uf593",
        "hotdog": "\uf80f",
        "hotel": "\uf594",
        "hourglass": "\uf254",
        "hourglass-2": "\uf254",
        "hourglass-half": "\uf254",
        "hourglass-empty": "\uf252",
        "hourglass-end": "\uf253",
        "hourglass-3": "\uf253",
        "hourglass-start": "\uf251",
        "hourglass-1": "\uf251",
        "house": "\uf015",
        "home": "\uf015",
        "home-alt": "\uf015",
        "home-lg-alt": "\uf015",
        "house-chimney": "\ue3af",
        "home-lg": "\ue3af",
        "house-chimney-crack": "\uf6f1",
        "house-damage": "\uf6f1",
        "house-chimney-medical": "\uf7f2",
        "clinic-medical": "\uf7f2",
        "house-chimney-user": "\ue065",
        "house-chimney-window": "\ue00d",
        "house-circle-check": "\ue509",
        "house-circle-exclamation": "\ue50a",
        "house-circle-xmark": "\ue50b",
        "house-crack": "\ue3b1",
        "house-fire": "\ue50c",
        "house-flag": "\ue50d",
        "house-flood-water": "\ue50e",
        "house-flood-water-circle-arrow-right": "\ue50f",
        "house-laptop": "\ue066",
        "laptop-house": "\ue066",
        "house-lock": "\ue510",
        "house-medical": "\ue3b2",
        "house-medical-circle-check": "\ue511",
        "house-medical-circle-exclamation": "\ue512",
        "house-medical-circle-xmark": "\ue513",
        "house-medical-flag": "\ue514",
        "house-signal": "\ue012",
        "house-tsunami": "\ue515",
        "house-user": "\ue1b0",
        "home-user": "\ue1b0",
        "hryvnia-sign": "\uf6f2",
        "hryvnia": "\uf6f2",
        "hurricane": "\uf751",
        "i": "I",
        "i-cursor": "\uf246",
        "ice-cream": "\uf810",
        "icicles": "\uf7ad",
        "icons": "\uf86d",
        "heart-music-camera-bolt": "\uf86d",
        "id-badge": "\uf2c1",
        "id-card": "\uf2c2",
        "drivers-license": "\uf2c2",
        "id-card-clip": "\uf47f",
        "id-card-alt": "\uf47f",
        "igloo": "\uf7ae",
        "image": "\uf03e",
        "image-portrait": "\uf3e0",
        "portrait": "\uf3e0",
        "images": "\uf302",
        "inbox": "\uf01c",
        "indent": "\uf03c",
        "indian-rupee-sign": "\ue1bc",
        "indian-rupee": "\ue1bc",
        "inr": "\ue1bc",
        "industry": "\uf275",
        "infinity": "\uf534",
        "info": "\uf129",
        "italic": "\uf033",
        "j": "J",
        "jar": "\ue516",
        "jar-wheat": "\ue517",
        "jedi": "\uf669",
        "jet-fighter": "\uf0fb",
        "fighter-jet": "\uf0fb",
        "jet-fighter-up": "\ue518",
        "joint": "\uf595",
        "jug-detergent": "\ue519",
        "k": "K",
        "kaaba": "\uf66b",
        "key": "\uf084",
        "keyboard": "\uf11c",
        "khanda": "\uf66d",
        "kip-sign": "\ue1c4",
        "kit-medical": "\uf479",
        "first-aid": "\uf479",
        "kitchen-set": "\ue51a",
        "kiwi-bird": "\uf535",
        "l": "L",
        "land-mine-on": "\ue51b",
        "landmark": "\uf66f",
        "landmark-dome": "\uf752",
        "landmark-alt": "\uf752",
        "landmark-flag": "\ue51c",
        "language": "\uf1ab",
        "laptop": "\uf109",
        "laptop-code": "\uf5fc",
        "laptop-file": "\ue51d",
        "laptop-medical": "\uf812",
        "lari-sign": "\ue1c8",
        "layer-group": "\uf5fd",
        "leaf": "\uf06c",
        "left-long": "\uf30a",
        "long-arrow-alt-left": "\uf30a",
        "left-right": "\uf337",
        "arrows-alt-h": "\uf337",
        "lemon": "\uf094",
        "less-than": "<",
        "less-than-equal": "\uf537",
        "life-ring": "\uf1cd",
        "lightbulb": "\uf0eb",
        "lines-leaning": "\ue51e",
        "link": "\uf0c1",
        "chain": "\uf0c1",
        "link-slash": "\uf127",
        "chain-broken": "\uf127",
        "chain-slash": "\uf127",
        "unlink": "\uf127",
        "lira-sign": "\uf195",
        "list": "\uf03a",
        "list-squares": "\uf03a",
        "list-check": "\uf0ae",
        "tasks": "\uf0ae",
        "list-ol": "\uf0cb",
        "list-1-2": "\uf0cb",
        "list-numeric": "\uf0cb",
        "list-ul": "\uf0ca",
        "list-dots": "\uf0ca",
        "litecoin-sign": "\ue1d3",
        "location-arrow": "\uf124",
        "location-crosshairs": "\uf601",
        "location": "\uf601",
        "location-dot": "\uf3c5",
        "map-marker-alt": "\uf3c5",
        "location-pin": "\uf041",
        "map-marker": "\uf041",
        "location-pin-lock": "\ue51f",
        "lock": "\uf023",
        "lock-open": "\uf3c1",
        "locust": "\ue520",
        "lungs": "\uf604",
        "lungs-virus": "\ue067",
        "m": "M",
        "magnet": "\uf076",
        "magnifying-glass": "\uf002",
        "search": "\uf002",
        "magnifying-glass-arrow-right": "\ue521",
        "magnifying-glass-chart": "\ue522",
        "magnifying-glass-dollar": "\uf688",
        "search-dollar": "\uf688",
        "magnifying-glass-location": "\uf689",
        "search-location": "\uf689",
        "magnifying-glass-minus": "\uf010",
        "search-minus": "\uf010",
        "magnifying-glass-plus": "\uf00e",
        "search-plus": "\uf00e",
        "manat-sign": "\ue1d5",
        "map": "\uf279",
        "map-location": "\uf59f",
        "map-marked": "\uf59f",
        "map-location-dot": "\uf5a0",
        "map-marked-alt": "\uf5a0",
        "map-pin": "\uf276",
        "marker": "\uf5a1",
        "mars": "\uf222",
        "mars-and-venus": "\uf224",
        "mars-and-venus-burst": "\ue523",
        "mars-double": "\uf227",
        "mars-stroke": "\uf229",
        "mars-stroke-right": "\uf22b",
        "mars-stroke-h": "\uf22b",
        "mars-stroke-up": "\uf22a",
        "mars-stroke-v": "\uf22a",
        "martini-glass": "\uf57b",
        "glass-martini-alt": "\uf57b",
        "martini-glass-citrus": "\uf561",
        "cocktail": "\uf561",
        "martini-glass-empty": "\uf000",
        "glass-martini": "\uf000",
        "mask": "\uf6fa",
        "mask-face": "\ue1d7",
        "mask-ventilator": "\ue524",
        "masks-theater": "\uf630",
        "theater-masks": "\uf630",
        "mattress-pillow": "\ue525",
        "maximize": "\uf31e",
        "expand-arrows-alt": "\uf31e",
        "medal": "\uf5a2",
        "memory": "\uf538",
        "menorah": "\uf676",
        "mercury": "\uf223",
        "message": "\uf27a",
        "comment-alt": "\uf27a",
        "meteor": "\uf753",
        "microchip": "\uf2db",
        "microphone": "\uf130",
        "microphone-lines": "\uf3c9",
        "microphone-alt": "\uf3c9",
        "microphone-lines-slash": "\uf539",
        "microphone-alt-slash": "\uf539",
        "microphone-slash": "\uf131",
        "microscope": "\uf610",
        "mill-sign": "\ue1ed",
        "minimize": "\uf78c",
        "compress-arrows-alt": "\uf78c",
        "minus": "\uf068",
        "subtract": "\uf068",
        "mitten": "\uf7b5",
        "mobile": "\uf3ce",
        "mobile-android": "\uf3ce",
        "mobile-phone": "\uf3ce",
        "mobile-button": "\uf10b",
        "mobile-retro": "\ue527",
        "mobile-screen": "\uf3cf",
        "mobile-android-alt": "\uf3cf",
        "mobile-screen-button": "\uf3cd",
        "mobile-alt": "\uf3cd",
        "money-bill": "\uf0d6",
        "money-bill-1": "\uf3d1",
        "money-bill-alt": "\uf3d1",
        "money-bill-1-wave": "\uf53b",
        "money-bill-wave-alt": "\uf53b",
        "money-bill-transfer": "\ue528",
        "money-bill-trend-up": "\ue529",
        "money-bill-wave": "\uf53a",
        "money-bill-wheat": "\ue52a",
        "money-bills": "\ue1f3",
        "money-check": "\uf53c",
        "money-check-dollar": "\uf53d",
        "money-check-alt": "\uf53d",
        "monument": "\uf5a6",
        "moon": "\uf186",
        "mortar-pestle": "\uf5a7",
        "mosque": "\uf678",
        "mosquito": "\ue52b",
        "mosquito-net": "\ue52c",
        "motorcycle": "\uf21c",
        "mound": "\ue52d",
        "mountain": "\uf6fc",
        "mountain-city": "\ue52e",
        "mountain-sun": "\ue52f",
        "mug-hot": "\uf7b6",
        "mug-saucer": "\uf0f4",
        "coffee": "\uf0f4",
        "music": "\uf001",
        "n": "N",
        "naira-sign": "\ue1f6",
        "network-wired": "\uf6ff",
        "neuter": "\uf22c",
        "newspaper": "\uf1ea",
        "not-equal": "\uf53e",
        "note-sticky": "\uf249",
        "sticky-note": "\uf249",
        "notes-medical": "\uf481",
        "o": "O",
        "object-group": "\uf247",
        "object-ungroup": "\uf248",
        "oil-can": "\uf613",
        "oil-well": "\ue532",
        "om": "\uf679",
        "otter": "\uf700",
        "outdent": "\uf03b",
        "dedent": "\uf03b",
        "p": "P",
        "pager": "\uf815",
        "paint-roller": "\uf5aa",
        "paintbrush": "\uf1fc",
        "paint-brush": "\uf1fc",
        "palette": "\uf53f",
        "pallet": "\uf482",
        "panorama": "\ue209",
        "paper-plane": "\uf1d8",
        "paperclip": "\uf0c6",
        "parachute-box": "\uf4cd",
        "paragraph": "\uf1dd",
        "passport": "\uf5ab",
        "paste": "\uf0ea",
        "file-clipboard": "\uf0ea",
        "pause": "\uf04c",
        "paw": "\uf1b0",
        "peace": "\uf67c",
        "pen": "\uf304",
        "pen-clip": "\uf305",
        "pen-alt": "\uf305",
        "pen-fancy": "\uf5ac",
        "pen-nib": "\uf5ad",
        "pen-ruler": "\uf5ae",
        "pencil-ruler": "\uf5ae",
        "pen-to-square": "\uf044",
        "edit": "\uf044",
        "pencil": "\uf303",
        "pencil-alt": "\uf303",
        "people-arrows-left-right": "\ue068",
        "people-arrows": "\ue068",
        "people-carry-box": "\uf4ce",
        "people-carry": "\uf4ce",
        "people-group": "\ue533",
        "people-line": "\ue534",
        "people-pulling": "\ue535",
        "people-robbery": "\ue536",
        "people-roof": "\ue537",
        "pepper-hot": "\uf816",
        "percent": "%",
        "percentage": "%",
        "person": "\uf183",
        "male": "\uf183",
        "person-arrow-down-to-line": "\ue538",
        "person-arrow-up-from-line": "\ue539",
        "person-biking": "\uf84a",
        "biking": "\uf84a",
        "person-booth": "\uf756",
        "person-breastfeeding": "\ue53a",
        "person-burst": "\ue53b",
        "person-cane": "\ue53c",
        "person-chalkboard": "\ue53d",
        "person-circle-check": "\ue53e",
        "person-circle-exclamation": "\ue53f",
        "person-circle-minus": "\ue540",
        "person-circle-plus": "\ue541",
        "person-circle-question": "\ue542",
        "person-circle-xmark": "\ue543",
        "person-digging": "\uf85e",
        "digging": "\uf85e",
        "person-dots-from-line": "\uf470",
        "diagnoses": "\uf470",
        "person-dress": "\uf182",
        "female": "\uf182",
        "person-dress-burst": "\ue544",
        "person-drowning": "\ue545",
        "person-falling": "\ue546",
        "person-falling-burst": "\ue547",
        "person-half-dress": "\ue548",
        "person-harassing": "\ue549",
        "person-hiking": "\uf6ec",
        "hiking": "\uf6ec",
        "person-military-pointing": "\ue54a",
        "person-military-rifle": "\ue54b",
        "person-military-to-person": "\ue54c",
        "person-praying": "\uf683",
        "pray": "\uf683",
        "person-pregnant": "\ue31e",
        "person-rays": "\ue54d",
        "person-rifle": "\ue54e",
        "person-running": "\uf70c",
        "running": "\uf70c",
        "person-shelter": "\ue54f",
        "person-skating": "\uf7c5",
        "skating": "\uf7c5",
        "person-skiing": "\uf7c9",
        "skiing": "\uf7c9",
        "person-skiing-nordic": "\uf7ca",
        "skiing-nordic": "\uf7ca",
        "person-snowboarding": "\uf7ce",
        "snowboarding": "\uf7ce",
        "person-swimming": "\uf5c4",
        "swimmer": "\uf5c4",
        "person-through-window": "\ue433",
        "person-walking": "\uf554",
        "walking": "\uf554",
        "person-walking-arrow-loop-left": "\ue551",
        "person-walking-arrow-right": "\ue552",
        "person-walking-dashed-line-arrow-right": "\ue553",
        "person-walking-luggage": "\ue554",
        "person-walking-with-cane": "\uf29d",
        "blind": "\uf29d",
        "peseta-sign": "\ue221",
        "peso-sign": "\ue222",
        "phone": "\uf095",
        "phone-flip": "\uf879",
        "phone-alt": "\uf879",
        "phone-slash": "\uf3dd",
        "phone-volume": "\uf2a0",
        "volume-control-phone": "\uf2a0",
        "photo-film": "\uf87c",
        "photo-video": "\uf87c",
        "piggy-bank": "\uf4d3",
        "pills": "\uf484",
        "pizza-slice": "\uf818",
        "place-of-worship": "\uf67f",
        "plane": "\uf072",
        "plane-arrival": "\uf5af",
        "plane-circle-check": "\ue555",
        "plane-circle-exclamation": "\ue556",
        "plane-circle-xmark": "\ue557",
        "plane-departure": "\uf5b0",
        "plane-lock": "\ue558",
        "plane-slash": "\ue069",
        "plane-up": "\ue22d",
        "plant-wilt": "\ue43b",
        "plate-wheat": "\ue55a",
        "play": "\uf04b",
        "plug": "\uf1e6",
        "plug-circle-bolt": "\ue55b",
        "plug-circle-check": "\ue55c",
        "plug-circle-exclamation": "\ue55d",
        "plug-circle-minus": "\ue55e",
        "plug-circle-plus": "\ue55f",
        "plug-circle-xmark": "\ue560",
        "plus": "+",
        "add": "+",
        "plus-minus": "\ue43c",
        "podcast": "\uf2ce",
        "poo": "\uf2fe",
        "poo-storm": "\uf75a",
        "poo-bolt": "\uf75a",
        "poop": "\uf619",
        "power-off": "\uf011",
        "prescription": "\uf5b1",
        "prescription-bottle": "\uf485",
        "prescription-bottle-medical": "\uf486",
        "prescription-bottle-alt": "\uf486",
        "print": "\uf02f",
        "pump-medical": "\ue06a",
        "pump-soap": "\ue06b",
        "puzzle-piece": "\uf12e",
        "q": "Q",
        "qrcode": "\uf029",
        "question": "?",
        "quote-left": "\uf10d",
        "quote-left-alt": "\uf10d",
        "quote-right": "\uf10e",
        "quote-right-alt": "\uf10e",
        "r": "R",
        "radiation": "\uf7b9",
        "radio": "\uf8d7",
        "rainbow": "\uf75b",
        "ranking-star": "\ue561",
        "receipt": "\uf543",
        "record-vinyl": "\uf8d9",
        "rectangle-ad": "\uf641",
        "ad": "\uf641",
        "rectangle-list": "\uf022",
        "list-alt": "\uf022",
        "rectangle-xmark": "\uf410",
        "rectangle-times": "\uf410",
        "times-rectangle": "\uf410",
        "window-close": "\uf410",
        "recycle": "\uf1b8",
        "registered": "\uf25d",
        "repeat": "\uf363",
        "reply": "\uf3e5",
        "mail-reply": "\uf3e5",
        "reply-all": "\uf122",
        "mail-reply-all": "\uf122",
        "republican": "\uf75e",
        "restroom": "\uf7bd",
        "retweet": "\uf079",
        "ribbon": "\uf4d6",
        "right-from-bracket": "\uf2f5",
        "sign-out-alt": "\uf2f5",
        "right-left": "\uf362",
        "exchange-alt": "\uf362",
        "right-long": "\uf30b",
        "long-arrow-alt-right": "\uf30b",
        "right-to-bracket": "\uf2f6",
        "sign-in-alt": "\uf2f6",
        "ring": "\uf70b",
        "road": "\uf018",
        "road-barrier": "\ue562",
        "road-bridge": "\ue563",
        "road-circle-check": "\ue564",
        "road-circle-exclamation": "\ue565",
        "road-circle-xmark": "\ue566",
        "road-lock": "\ue567",
        "road-spikes": "\ue568",
        "robot": "\uf544",
        "rocket": "\uf135",
        "rotate": "\uf2f1",
        "sync-alt": "\uf2f1",
        "rotate-left": "\uf2ea",
        "rotate-back": "\uf2ea",
        "rotate-backward": "\uf2ea",
        "undo-alt": "\uf2ea",
        "rotate-right": "\uf2f9",
        "redo-alt": "\uf2f9",
        "rotate-forward": "\uf2f9",
        "route": "\uf4d7",
        "rss": "\uf09e",
        "feed": "\uf09e",
        "ruble-sign": "\uf158",
        "rouble": "\uf158",
        "rub": "\uf158",
        "ruble": "\uf158",
        "rug": "\ue569",
        "ruler": "\uf545",
        "ruler-combined": "\uf546",
        "ruler-horizontal": "\uf547",
        "ruler-vertical": "\uf548",
        "rupee-sign": "\uf156",
        "rupee": "\uf156",
        "rupiah-sign": "\ue23d",
        "s": "S",
        "sack-dollar": "\uf81d",
        "sack-xmark": "\ue56a",
        "sailboat": "\ue445",
        "satellite": "\uf7bf",
        "satellite-dish": "\uf7c0",
        "scale-balanced": "\uf24e",
        "balance-scale": "\uf24e",
        "scale-unbalanced": "\uf515",
        "balance-scale-left": "\uf515",
        "scale-unbalanced-flip": "\uf516",
        "balance-scale-right": "\uf516",
        "school": "\uf549",
        "school-circle-check": "\ue56b",
        "school-circle-exclamation": "\ue56c",
        "school-circle-xmark": "\ue56d",
        "school-flag": "\ue56e",
        "school-lock": "\ue56f",
        "scissors": "\uf0c4",
        "cut": "\uf0c4",
        "screwdriver": "\uf54a",
        "screwdriver-wrench": "\uf7d9",
        "tools": "\uf7d9",
        "scroll": "\uf70e",
        "scroll-torah": "\uf6a0",
        "torah": "\uf6a0",
        "sd-card": "\uf7c2",
        "section": "\ue447",
        "seedling": "\uf4d8",
        "sprout": "\uf4d8",
        "server": "\uf233",
        "shapes": "\uf61f",
        "triangle-circle-square": "\uf61f",
        "share": "\uf064",
        "arrow-turn-right": "\uf064",
        "mail-forward": "\uf064",
        "share-from-square": "\uf14d",
        "share-square": "\uf14d",
        "share-nodes": "\uf1e0",
        "share-alt": "\uf1e0",
        "sheet-plastic": "\ue571",
        "shekel-sign": "\uf20b",
        "ils": "\uf20b",
        "shekel": "\uf20b",
        "sheqel": "\uf20b",
        "sheqel-sign": "\uf20b",
        "shield": "\uf132",
        "shield-blank": "\uf132",
        "shield-cat": "\ue572",
        "shield-dog": "\ue573",
        "shield-halved": "\uf3ed",
        "shield-alt": "\uf3ed",
        "shield-heart": "\ue574",
        "shield-virus": "\ue06c",
        "ship": "\uf21a",
        "shirt": "\uf553",
        "t-shirt": "\uf553",
        "tshirt": "\uf553",
        "shoe-prints": "\uf54b",
        "shop": "\uf54f",
        "store-alt": "\uf54f",
        "shop-lock": "\ue4a5",
        "shop-slash": "\ue070",
        "store-alt-slash": "\ue070",
        "shower": "\uf2cc",
        "shrimp": "\ue448",
        "shuffle": "\uf074",
        "random": "\uf074",
        "shuttle-space": "\uf197",
        "space-shuttle": "\uf197",
        "sign-hanging": "\uf4d9",
        "sign": "\uf4d9",
        "signal": "\uf012",
        "signal-5": "\uf012",
        "signal-perfect": "\uf012",
        "signature": "\uf5b7",
        "signs-post": "\uf277",
        "map-signs": "\uf277",
        "sim-card": "\uf7c4",
        "sink": "\ue06d",
        "sitemap": "\uf0e8",
        "skull": "\uf54c",
        "skull-crossbones": "\uf714",
        "slash": "\uf715",
        "sleigh": "\uf7cc",
        "sliders": "\uf1de",
        "sliders-h": "\uf1de",
        "smog": "\uf75f",
        "smoking": "\uf48d",
        "snowflake": "\uf2dc",
        "snowman": "\uf7d0",
        "snowplow": "\uf7d2",
        "soap": "\ue06e",
        "socks": "\uf696",
        "solar-panel": "\uf5ba",
        "sort": "\uf0dc",
        "unsorted": "\uf0dc",
        "sort-down": "\uf0dd",
        "sort-desc": "\uf0dd",
        "sort-up": "\uf0de",
        "sort-asc": "\uf0de",
        "spa": "\uf5bb",
        "spaghetti-monster-flying": "\uf67b",
        "pastafarianism": "\uf67b",
        "spell-check": "\uf891",
        "spider": "\uf717",
        "spinner": "\uf110",
        "splotch": "\uf5bc",
        "spoon": "\uf2e5",
        "utensil-spoon": "\uf2e5",
        "spray-can": "\uf5bd",
        "spray-can-sparkles": "\uf5d0",
        "air-freshener": "\uf5d0",
        "square": "\uf0c8",
        "square-arrow-up-right": "\uf14c",
        "external-link-square": "\uf14c",
        "square-caret-down": "\uf150",
        "caret-square-down": "\uf150",
        "square-caret-left": "\uf191",
        "caret-square-left": "\uf191",
        "square-caret-right": "\uf152",
        "caret-square-right": "\uf152",
        "square-caret-up": "\uf151",
        "caret-square-up": "\uf151",
        "square-check": "\uf14a",
        "check-square": "\uf14a",
        "square-envelope": "\uf199",
        "envelope-square": "\uf199",
        "square-full": "\uf45c",
        "square-h": "\uf0fd",
        "h-square": "\uf0fd",
        "square-minus": "\uf146",
        "minus-square": "\uf146",
        "square-nfi": "\ue576",
        "square-parking": "\uf540",
        "parking": "\uf540",
        "square-pen": "\uf14b",
        "pen-square": "\uf14b",
        "pencil-square": "\uf14b",
        "square-person-confined": "\ue577",
        "square-phone": "\uf098",
        "phone-square": "\uf098",
        "square-phone-flip": "\uf87b",
        "phone-square-alt": "\uf87b",
        "square-plus": "\uf0fe",
        "plus-square": "\uf0fe",
        "square-poll-horizontal": "\uf682",
        "poll-h": "\uf682",
        "square-poll-vertical": "\uf681",
        "poll": "\uf681",
        "square-root-variable": "\uf698",
        "square-root-alt": "\uf698",
        "square-rss": "\uf143",
        "rss-square": "\uf143",
        "square-share-nodes": "\uf1e1",
        "share-alt-square": "\uf1e1",
        "square-up-right": "\uf360",
        "external-link-square-alt": "\uf360",
        "square-virus": "\ue578",
        "square-xmark": "\uf2d3",
        "times-square": "\uf2d3",
        "xmark-square": "\uf2d3",
        "staff-aesculapius": "\ue579",
        "rod-asclepius": "\ue579",
        "rod-snake": "\ue579",
        "staff-snake": "\ue579",
        "stairs": "\ue289",
        "stamp": "\uf5bf",
        "star": "\uf005",
        "star-and-crescent": "\uf699",
        "star-half": "\uf089",
        "star-half-stroke": "\uf5c0",
        "star-half-alt": "\uf5c0",
        "star-of-david": "\uf69a",
        "star-of-life": "\uf621",
        "sterling-sign": "\uf154",
        "gbp": "\uf154",
        "pound-sign": "\uf154",
        "stethoscope": "\uf0f1",
        "stop": "\uf04d",
        "stopwatch": "\uf2f2",
        "stopwatch-20": "\ue06f",
        "store": "\uf54e",
        "store-slash": "\ue071",
        "street-view": "\uf21d",
        "strikethrough": "\uf0cc",
        "stroopwafel": "\uf551",
        "subscript": "\uf12c",
        "suitcase": "\uf0f2",
        "suitcase-medical": "\uf0fa",
        "medkit": "\uf0fa",
        "suitcase-rolling": "\uf5c1",
        "sun": "\uf185",
        "sun-plant-wilt": "\ue57a",
        "superscript": "\uf12b",
        "swatchbook": "\uf5c3",
        "synagogue": "\uf69b",
        "syringe": "\uf48e",
        "t": "T",
        "table": "\uf0ce",
        "table-cells": "\uf00a",
        "th": "\uf00a",
        "table-cells-large": "\uf009",
        "th-large": "\uf009",
        "table-columns": "\uf0db",
        "columns": "\uf0db",
        "table-list": "\uf00b",
        "th-list": "\uf00b",
        "table-tennis-paddle-ball": "\uf45d",
        "ping-pong-paddle-ball": "\uf45d",
        "table-tennis": "\uf45d",
        "tablet": "\uf3fb",
        "tablet-android": "\uf3fb",
        "tablet-button": "\uf10a",
        "tablet-screen-button": "\uf3fa",
        "tablet-alt": "\uf3fa",
        "tablets": "\uf490",
        "tachograph-digital": "\uf566",
        "digital-tachograph": "\uf566",
        "tag": "\uf02b",
        "tags": "\uf02c",
        "tape": "\uf4db",
        "tarp": "\ue57b",
        "tarp-droplet": "\ue57c",
        "taxi": "\uf1ba",
        "cab": "\uf1ba",
        "teeth": "\uf62e",
        "teeth-open": "\uf62f",
        "temperature-arrow-down": "\ue03f",
        "temperature-down": "\ue03f",
        "temperature-arrow-up": "\ue040",
        "temperature-up": "\ue040",
        "temperature-empty": "\uf2cb",
        "temperature-0": "\uf2cb",
        "thermometer-0": "\uf2cb",
        "thermometer-empty": "\uf2cb",
        "temperature-full": "\uf2c7",
        "temperature-4": "\uf2c7",
        "thermometer-4": "\uf2c7",
        "thermometer-full": "\uf2c7",
        "temperature-half": "\uf2c9",
        "temperature-2": "\uf2c9",
        "thermometer-2": "\uf2c9",
        "thermometer-half": "\uf2c9",
        "temperature-high": "\uf769",
        "temperature-low": "\uf76b",
        "temperature-quarter": "\uf2ca",
        "temperature-1": "\uf2ca",
        "thermometer-1": "\uf2ca",
        "thermometer-quarter": "\uf2ca",
        "temperature-three-quarters": "\uf2c8",
        "temperature-3": "\uf2c8",
        "thermometer-3": "\uf2c8",
        "thermometer-three-quarters": "\uf2c8",
        "tenge-sign": "\uf7d7",
        "tenge": "\uf7d7",
        "tent": "\ue57d",
        "tent-arrow-down-to-line": "\ue57e",
        "tent-arrow-left-right": "\ue57f",
        "tent-arrow-turn-left": "\ue580",
        "tent-arrows-down": "\ue581",
        "tents": "\ue582",
        "terminal": "\uf120",
        "text-height": "\uf034",
        "text-slash": "\uf87d",
        "remove-format": "\uf87d",
        "text-width": "\uf035",
        "thermometer": "\uf491",
        "thumbs-down": "\uf165",
        "thumbs-up": "\uf164",
        "thumbtack": "\uf08d",
        "thumb-tack": "\uf08d",
        "ticket": "\uf145",
        "ticket-simple": "\uf3ff",
        "ticket-alt": "\uf3ff",
        "timeline": "\ue29c",
        "toggle-off": "\uf204",
        "toggle-on": "\uf205",
        "toilet": "\uf7d8",
        "toilet-paper": "\uf71e",
        "toilet-paper-slash": "\ue072",
        "toilet-portable": "\ue583",
        "toilets-portable": "\ue584",
        "toolbox": "\uf552",
        "tooth": "\uf5c9",
        "torii-gate": "\uf6a1",
        "tornado": "\uf76f",
        "tower-broadcast": "\uf519",
        "broadcast-tower": "\uf519",
        "tower-cell": "\ue585",
        "tower-observation": "\ue586",
        "tractor": "\uf722",
        "trademark": "\uf25c",
        "traffic-light": "\uf637",
        "trailer": "\ue041",
        "train": "\uf238",
        "train-subway": "\uf239",
        "subway": "\uf239",
        "train-tram": "\uf7da",
        "tram": "\uf7da",
        "transgender": "\uf225",
        "transgender-alt": "\uf225",
        "trash": "\uf1f8",
        "trash-arrow-up": "\uf829",
        "trash-restore": "\uf829",
        "trash-can": "\uf2ed",
        "trash-alt": "\uf2ed",
        "trash-can-arrow-up": "\uf82a",
        "trash-restore-alt": "\uf82a",
        "tree": "\uf1bb",
        "tree-city": "\ue587",
        "triangle-exclamation": "\uf071",
        "exclamation-triangle": "\uf071",
        "warning": "\uf071",
        "trophy": "\uf091",
        "trowel": "\ue589",
        "trowel-bricks": "\ue58a",
        "truck": "\uf0d1",
        "truck-arrow-right": "\ue58b",
        "truck-droplet": "\ue58c",
        "truck-fast": "\uf48b",
        "shipping-fast": "\uf48b",
        "truck-field": "\ue58d",
        "truck-field-un": "\ue58e",
        "truck-front": "\ue2b7",
        "truck-medical": "\uf0f9",
        "ambulance": "\uf0f9",
        "truck-monster": "\uf63b",
        "truck-moving": "\uf4df",
        "truck-pickup": "\uf63c",
        "truck-plane": "\ue58f",
        "truck-ramp-box": "\uf4de",
        "truck-loading": "\uf4de",
        "tty": "\uf1e4",
        "teletype": "\uf1e4",
        "turkish-lira-sign": "\ue2bb",
        "try": "\ue2bb",
        "turkish-lira": "\ue2bb",
        "turn-down": "\uf3be",
        "level-down-alt": "\uf3be",
        "turn-up": "\uf3bf",
        "level-up-alt": "\uf3bf",
        "tv": "\uf26c",
        "television": "\uf26c",
        "tv-alt": "\uf26c",
        "u": "U",
        "umbrella": "\uf0e9",
        "umbrella-beach": "\uf5ca",
        "underline": "\uf0cd",
        "universal-access": "\uf29a",
        "unlock": "\uf09c",
        "unlock-keyhole": "\uf13e",
        "unlock-alt": "\uf13e",
        "up-down": "\uf338",
        "arrows-alt-v": "\uf338",
        "up-down-left-right": "\uf0b2",
        "arrows-alt": "\uf0b2",
        "up-long": "\uf30c",
        "long-arrow-alt-up": "\uf30c",
        "up-right-and-down-left-from-center": "\uf424",
        "expand-alt": "\uf424",
        "up-right-from-square": "\uf35d",
        "external-link-alt": "\uf35d",
        "upload": "\uf093",
        "user": "\uf007",
        "user-astronaut": "\uf4fb",
        "user-check": "\uf4fc",
        "user-clock": "\uf4fd",
        "user-doctor": "\uf0f0",
        "user-md": "\uf0f0",
        "user-gear": "\uf4fe",
        "user-cog": "\uf4fe",
        "user-graduate": "\uf501",
        "user-group": "\uf500",
        "user-friends": "\uf500",
        "user-injured": "\uf728",
        "user-large": "\uf406",
        "user-alt": "\uf406",
        "user-large-slash": "\uf4fa",
        "user-alt-slash": "\uf4fa",
        "user-lock": "\uf502",
        "user-minus": "\uf503",
        "user-ninja": "\uf504",
        "user-nurse": "\uf82f",
        "user-pen": "\uf4ff",
        "user-edit": "\uf4ff",
        "user-plus": "\uf234",
        "user-secret": "\uf21b",
        "user-shield": "\uf505",
        "user-slash": "\uf506",
        "user-tag": "\uf507",
        "user-tie": "\uf508",
        "user-xmark": "\uf235",
        "user-times": "\uf235",
        "users": "\uf0c0",
        "users-between-lines": "\ue591",
        "users-gear": "\uf509",
        "users-cog": "\uf509",
        "users-line": "\ue592",
        "users-rays": "\ue593",
        "users-rectangle": "\ue594",
        "users-slash": "\ue073",
        "users-viewfinder": "\ue595",
        "utensils": "\uf2e7",
        "cutlery": "\uf2e7",
        "v": "V",
        "van-shuttle": "\uf5b6",
        "shuttle-van": "\uf5b6",
        "vault": "\ue2c5",
        "vector-square": "\uf5cb",
        "venus": "\uf221",
        "venus-double": "\uf226",
        "venus-mars": "\uf228",
        "vest": "\ue085",
        "vest-patches": "\ue086",
        "vial": "\uf492",
        "vial-circle-check": "\ue596",
        "vial-virus": "\ue597",
        "vials": "\uf493",
        "video": "\uf03d",
        "video-camera": "\uf03d",
        "video-slash": "\uf4e2",
        "vihara": "\uf6a7",
        "virus": "\ue074",
        "virus-covid": "\ue4a8",
        "virus-covid-slash": "\ue4a9",
        "virus-slash": "\ue075",
        "viruses": "\ue076",
        "voicemail": "\uf897",
        "volcano": "\uf770",
        "volleyball": "\uf45f",
        "volleyball-ball": "\uf45f",
        "volume-high": "\uf028",
        "volume-up": "\uf028",
        "volume-low": "\uf027",
        "volume-down": "\uf027",
        "volume-off": "\uf026",
        "volume-xmark": "\uf6a9",
        "volume-mute": "\uf6a9",
        "volume-times": "\uf6a9",
        "vr-cardboard": "\uf729",
        "w": "W",
        "walkie-talkie": "\uf8ef",
        "wallet": "\uf555",
        "wand-magic": "\uf0d0",
        "magic": "\uf0d0",
        "wand-magic-sparkles": "\ue2ca",
        "magic-wand-sparkles": "\ue2ca",
        "wand-sparkles": "\uf72b",
        "warehouse": "\uf494",
        "water": "\uf773",
        "water-ladder": "\uf5c5",
        "ladder-water": "\uf5c5",
        "swimming-pool": "\uf5c5",
        "wave-square": "\uf83e",
        "weight-hanging": "\uf5cd",
        "weight-scale": "\uf496",
        "weight": "\uf496",
        "wheat-awn": "\ue2cd",
        "wheat-alt": "\ue2cd",
        "wheat-awn-circle-exclamation": "\ue598",
        "wheelchair": "\uf193",
        "wheelchair-move": "\ue2ce",
        "wheelchair-alt": "\ue2ce",
        "whiskey-glass": "\uf7a0",
        "glass-whiskey": "\uf7a0",
        "wifi": "\uf1eb",
        "wifi-3": "\uf1eb",
        "wifi-strong": "\uf1eb",
        "wind": "\uf72e",
        "window-maximize": "\uf2d0",
        "window-minimize": "\uf2d1",
        "window-restore": "\uf2d2",
        "wine-bottle": "\uf72f",
        "wine-glass": "\uf4e3",
        "wine-glass-empty": "\uf5ce",
        "wine-glass-alt": "\uf5ce",
        "won-sign": "\uf159",
        "krw": "\uf159",
        "won": "\uf159",
        "worm": "\ue599",
        "wrench": "\uf0ad",
        "x": "X",
        "x-ray": "\uf497",
        "xmark": "\uf00d",
        "close": "\uf00d",
        "multiply": "\uf00d",
        "remove": "\uf00d",
        "times": "\uf00d",
        "xmarks-lines": "\ue59a",
        "y": "Y",
        "yen-sign": "\uf157",
        "cny": "\uf157",
        "jpy": "\uf157",
        "rmb": "\uf157",
        "yen": "\uf157",
        "yin-yang": "\uf6ad",
        "z": "Z"
    },
    "brands": {
        "42-group": "\ue080",
        "innosoft": "\ue080",
        "500px": "\uf26e",
        "accessible-icon": "\uf368",
        "accusoft": "\uf369",
        "adn": "\uf170",
        "adversal": "\uf36a",
        "affiliatetheme": "\uf36b",
        "airbnb": "\uf834",
        "algolia": "\uf36c",
        "alipay": "\uf642",
        "amazon": "\uf270",
        "amazon-pay": "\uf42c",
        "amilia": "\uf36d",
        "android": "\uf17b",
        "angellist": "\uf209",
        "angrycreative": "\uf36e",
        "angular": "\uf420",
        "app-store": "\uf36f",
        "app-store-ios": "\uf370",
        "apper": "\uf371",
        "apple": "\uf179",
        "apple-pay": "\uf415",
        "artstation": "\uf77a",
        "asymmetrik": "\uf372",
        "atlassian": "\uf77b",
        "audible": "\uf373",
        "autoprefixer": "\uf41c",
        "avianex": "\uf374",
        "aviato": "\uf421",
        "aws": "\uf375",
        "bandcamp": "\uf2d5",
        "battle-net": "\uf835",
        "behance": "\uf1b4",
        "behance-square": "\uf1b5",
        "bilibili": "\ue3d9",
        "bimobject": "\uf378",
        "bitbucket": "\uf171",
        "bitcoin": "\uf379",
        "bity": "\uf37a",
        "black-tie": "\uf27e",
        "blackberry": "\uf37b",
        "blogger": "\uf37c",
        "blogger-b": "\uf37d",
        "bluetooth": "\uf293",
        "bluetooth-b": "\uf294",
        "bootstrap": "\uf836",
        "bots": "\ue340",
        "btc": "\uf15a",
        "buffer": "\uf837",
        "buromobelexperte": "\uf37f",
        "buy-n-large": "\uf8a6",
        "buysellads": "\uf20d",
        "canadian-maple-leaf": "\uf785",
        "cc-amazon-pay": "\uf42d",
        "cc-amex": "\uf1f3",
        "cc-apple-pay": "\uf416",
        "cc-diners-club": "\uf24c",
        "cc-discover": "\uf1f2",
        "cc-jcb": "\uf24b",
        "cc-mastercard": "\uf1f1",
        "cc-paypal": "\uf1f4",
        "cc-stripe": "\uf1f5",
        "cc-visa": "\uf1f0",
        "centercode": "\uf380",
        "centos": "\uf789",
        "chrome": "\uf268",
        "chromecast": "\uf838",
        "cloudflare": "\ue07d",
        "cloudscale": "\uf383",
        "cloudsmith": "\uf384",
        "cloudversify": "\uf385",
        "cmplid": "\ue360",
        "codepen": "\uf1cb",
        "codiepie": "\uf284",
        "confluence": "\uf78d",
        "connectdevelop": "\uf20e",
        "contao": "\uf26d",
        "cotton-bureau": "\uf89e",
        "cpanel": "\uf388",
        "creative-commons": "\uf25e",
        "creative-commons-by": "\uf4e7",
        "creative-commons-nc": "\uf4e8",
        "creative-commons-nc-eu": "\uf4e9",
        "creative-commons-nc-jp": "\uf4ea",
        "creative-commons-nd": "\uf4eb",
        "creative-commons-pd": "\uf4ec",
        "creative-commons-pd-alt": "\uf4ed",
        "creative-commons-remix": "\uf4ee",
        "creative-commons-sa": "\uf4ef",
        "creative-commons-sampling": "\uf4f0",
        "creative-commons-sampling-plus": "\uf4f1",
        "creative-commons-share": "\uf4f2",
        "creative-commons-zero": "\uf4f3",
        "critical-role": "\uf6c9",
        "css3": "\uf13c",
        "css3-alt": "\uf38b",
        "cuttlefish": "\uf38c",
        "d-and-d": "\uf38d",
        "d-and-d-beyond": "\uf6ca",
        "dailymotion": "\ue052",
        "dashcube": "\uf210",
        "deezer": "\ue077",
        "delicious": "\uf1a5",
        "deploydog": "\uf38e",
        "deskpro": "\uf38f",
        "dev": "\uf6cc",
        "deviantart": "\uf1bd",
        "dhl": "\uf790",
        "diaspora": "\uf791",
        "digg": "\uf1a6",
        "digital-ocean": "\uf391",
        "discord": "\uf392",
        "discourse": "\uf393",
        "dochub": "\uf394",
        "docker": "\uf395",
        "draft2digital": "\uf396",
        "dribbble": "\uf17d",
        "dribbble-square": "\uf397",
        "dropbox": "\uf16b",
        "drupal": "\uf1a9",
        "dyalog": "\uf399",
        "earlybirds": "\uf39a",
        "ebay": "\uf4f4",
        "edge": "\uf282",
        "edge-legacy": "\ue078",
        "elementor": "\uf430",
        "ello": "\uf5f1",
        "ember": "\uf423",
        "empire": "\uf1d1",
        "envira": "\uf299",
        "erlang": "\uf39d",
        "ethereum": "\uf42e",
        "etsy": "\uf2d7",
        "evernote": "\uf839",
        "expeditedssl": "\uf23e",
        "facebook": "\uf09a",
        "facebook-f": "\uf39e",
        "facebook-messenger": "\uf39f",
        "facebook-square": "\uf082",
        "fantasy-flight-games": "\uf6dc",
        "fedex": "\uf797",
        "fedora": "\uf798",
        "figma": "\uf799",
        "firefox": "\uf269",
        "firefox-browser": "\ue007",
        "first-order": "\uf2b0",
        "first-order-alt": "\uf50a",
        "firstdraft": "\uf3a1",
        "flickr": "\uf16e",
        "flipboard": "\uf44d",
        "fly": "\uf417",
        "font-awesome": "\uf2b4",
        "font-awesome-flag": "\uf2b4",
        "font-awesome-logo-full": "\uf2b4",
        "fonticons": "\uf280",
        "fonticons-fi": "\uf3a2",
        "fort-awesome": "\uf286",
        "fort-awesome-alt": "\uf3a3",
        "forumbee": "\uf211",
        "foursquare": "\uf180",
        "free-code-camp": "\uf2c5",
        "freebsd": "\uf3a4",
        "fulcrum": "\uf50b",
        "galactic-republic": "\uf50c",
        "galactic-senate": "\uf50d",
        "get-pocket": "\uf265",
        "gg": "\uf260",
        "gg-circle": "\uf261",
        "git": "\uf1d3",
        "git-alt": "\uf841",
        "git-square": "\uf1d2",
        "github": "\uf09b",
        "github-alt": "\uf113",
        "github-square": "\uf092",
        "gitkraken": "\uf3a6",
        "gitlab": "\uf296",
        "gitter": "\uf426",
        "glide": "\uf2a5",
        "glide-g": "\uf2a6",
        "gofore": "\uf3a7",
        "golang": "\ue40f",
        "goodreads": "\uf3a8",
        "goodreads-g": "\uf3a9",
        "google": "\uf1a0",
        "google-drive": "\uf3aa",
        "google-pay": "\ue079",
        "google-play": "\uf3ab",
        "google-plus": "\uf2b3",
        "google-plus-g": "\uf0d5",
        "google-plus-square": "\uf0d4",
        "google-wallet": "\uf1ee",
        "gratipay": "\uf184",
        "grav": "\uf2d6",
        "gripfire": "\uf3ac",
        "grunt": "\uf3ad",
        "guilded": "\ue07e",
        "gulp": "\uf3ae",
        "hacker-news": "\uf1d4",
        "hacker-news-square": "\uf3af",
        "hackerrank": "\uf5f7",
        "hashnode": "\ue499",
        "hips": "\uf452",
        "hire-a-helper": "\uf3b0",
        "hive": "\ue07f",
        "hooli": "\uf427",
        "hornbill": "\uf592",
        "hotjar": "\uf3b1",
        "houzz": "\uf27c",
        "html5": "\uf13b",
        "hubspot": "\uf3b2",
        "ideal": "\ue013",
        "imdb": "\uf2d8",
        "instagram": "\uf16d",
        "instagram-square": "\ue055",
        "instalod": "\ue081",
        "intercom": "\uf7af",
        "internet-explorer": "\uf26b",
        "invision": "\uf7b0",
        "ioxhost": "\uf208",
        "itch-io": "\uf83a",
        "itunes": "\uf3b4",
        "itunes-note": "\uf3b5",
        "java": "\uf4e4",
        "jedi-order": "\uf50e",
        "jenkins": "\uf3b6",
        "jira": "\uf7b1",
        "joget": "\uf3b7",
        "joomla": "\uf1aa",
        "js": "\uf3b8",
        "js-square": "\uf3b9",
        "jsfiddle": "\uf1cc",
        "kaggle": "\uf5fa",
        "keybase": "\uf4f5",
        "keycdn": "\uf3ba",
        "kickstarter": "\uf3bb",
        "kickstarter-k": "\uf3bc",
        "korvue": "\uf42f",
        "laravel": "\uf3bd",
        "lastfm": "\uf202",
        "lastfm-square": "\uf203",
        "leanpub": "\uf212",
        "less": "\uf41d",
        "line": "\uf3c0",
        "linkedin": "\uf08c",
        "linkedin-in": "\uf0e1",
        "linode": "\uf2b8",
        "linux": "\uf17c",
        "lyft": "\uf3c3",
        "magento": "\uf3c4",
        "mailchimp": "\uf59e",
        "mandalorian": "\uf50f",
        "markdown": "\uf60f",
        "mastodon": "\uf4f6",
        "maxcdn": "\uf136",
        "mdb": "\uf8ca",
        "medapps": "\uf3c6",
        "medium": "\uf23a",
        "medium-m": "\uf23a",
        "medrt": "\uf3c8",
        "meetup": "\uf2e0",
        "megaport": "\uf5a3",
        "mendeley": "\uf7b3",
        "microblog": "\ue01a",
        "microsoft": "\uf3ca",
        "mix": "\uf3cb",
        "mixcloud": "\uf289",
        "mixer": "\ue056",
        "mizuni": "\uf3cc",
        "modx": "\uf285",
        "monero": "\uf3d0",
        "napster": "\uf3d2",
        "neos": "\uf612",
        "nfc-directional": "\ue530",
        "nfc-symbol": "\ue531",
        "nimblr": "\uf5a8",
        "node": "\uf419",
        "node-js": "\uf3d3",
        "npm": "\uf3d4",
        "ns8": "\uf3d5",
        "nutritionix": "\uf3d6",
        "octopus-deploy": "\ue082",
        "odnoklassniki": "\uf263",
        "odnoklassniki-square": "\uf264",
        "old-republic": "\uf510",
        "opencart": "\uf23d",
        "openid": "\uf19b",
        "opera": "\uf26a",
        "optin-monster": "\uf23c",
        "orcid": "\uf8d2",
        "osi": "\uf41a",
        "padlet": "\ue4a0",
        "page4": "\uf3d7",
        "pagelines": "\uf18c",
        "palfed": "\uf3d8",
        "patreon": "\uf3d9",
        "paypal": "\uf1ed",
        "perbyte": "\ue083",
        "periscope": "\uf3da",
        "phabricator": "\uf3db",
        "phoenix-framework": "\uf3dc",
        "phoenix-squadron": "\uf511",
        "php": "\uf457",
        "pied-piper": "\uf2ae",
        "pied-piper-alt": "\uf1a8",
        "pied-piper-hat": "\uf4e5",
        "pied-piper-pp": "\uf1a7",
        "pied-piper-square": "\ue01e",
        "pinterest": "\uf0d2",
        "pinterest-p": "\uf231",
        "pinterest-square": "\uf0d3",
        "pix": "\ue43a",
        "playstation": "\uf3df",
        "product-hunt": "\uf288",
        "pushed": "\uf3e1",
        "python": "\uf3e2",
        "qq": "\uf1d6",
        "quinscape": "\uf459",
        "quora": "\uf2c4",
        "r-project": "\uf4f7",
        "raspberry-pi": "\uf7bb",
        "ravelry": "\uf2d9",
        "react": "\uf41b",
        "reacteurope": "\uf75d",
        "readme": "\uf4d5",
        "rebel": "\uf1d0",
        "red-river": "\uf3e3",
        "reddit": "\uf1a1",
        "reddit-alien": "\uf281",
        "reddit-square": "\uf1a2",
        "redhat": "\uf7bc",
        "renren": "\uf18b",
        "replyd": "\uf3e6",
        "researchgate": "\uf4f8",
        "resolving": "\uf3e7",
        "rev": "\uf5b2",
        "rocketchat": "\uf3e8",
        "rockrms": "\uf3e9",
        "rust": "\ue07a",
        "safari": "\uf267",
        "salesforce": "\uf83b",
        "sass": "\uf41e",
        "schlix": "\uf3ea",
        "screenpal": "\ue570",
        "scribd": "\uf28a",
        "searchengin": "\uf3eb",
        "sellcast": "\uf2da",
        "sellsy": "\uf213",
        "servicestack": "\uf3ec",
        "shirtsinbulk": "\uf214",
        "shopify": "\ue057",
        "shopware": "\uf5b5",
        "simplybuilt": "\uf215",
        "sistrix": "\uf3ee",
        "sith": "\uf512",
        "sitrox": "\ue44a",
        "sketch": "\uf7c6",
        "skyatlas": "\uf216",
        "skype": "\uf17e",
        "slack": "\uf198",
        "slack-hash": "\uf198",
        "slideshare": "\uf1e7",
        "snapchat": "\uf2ab",
        "snapchat-ghost": "\uf2ab",
        "snapchat-square": "\uf2ad",
        "soundcloud": "\uf1be",
        "sourcetree": "\uf7d3",
        "speakap": "\uf3f3",
        "speaker-deck": "\uf83c",
        "spotify": "\uf1bc",
        "square-font-awesome": "\uf425",
        "square-font-awesome-stroke": "\uf35c",
        "font-awesome-alt": "\uf35c",
        "squarespace": "\uf5be",
        "stack-exchange": "\uf18d",
        "stack-overflow": "\uf16c",
        "stackpath": "\uf842",
        "staylinked": "\uf3f5",
        "steam": "\uf1b6",
        "steam-square": "\uf1b7",
        "steam-symbol": "\uf3f6",
        "sticker-mule": "\uf3f7",
        "strava": "\uf428",
        "stripe": "\uf429",
        "stripe-s": "\uf42a",
        "studiovinari": "\uf3f8",
        "stumbleupon": "\uf1a4",
        "stumbleupon-circle": "\uf1a3",
        "superpowers": "\uf2dd",
        "supple": "\uf3f9",
        "suse": "\uf7d6",
        "swift": "\uf8e1",
        "symfony": "\uf83d",
        "teamspeak": "\uf4f9",
        "telegram": "\uf2c6",
        "telegram-plane": "\uf2c6",
        "tencent-weibo": "\uf1d5",
        "the-red-yeti": "\uf69d",
        "themeco": "\uf5c6",
        "themeisle": "\uf2b2",
        "think-peaks": "\uf731",
        "tiktok": "\ue07b",
        "trade-federation": "\uf513",
        "trello": "\uf181",
        "tumblr": "\uf173",
        "tumblr-square": "\uf174",
        "twitch": "\uf1e8",
        "twitter": "\uf099",
        "twitter-square": "\uf081",
        "typo3": "\uf42b",
        "uber": "\uf402",
        "ubuntu": "\uf7df",
        "uikit": "\uf403",
        "umbraco": "\uf8e8",
        "uncharted": "\ue084",
        "uniregistry": "\uf404",
        "unity": "\ue049",
        "unsplash": "\ue07c",
        "untappd": "\uf405",
        "ups": "\uf7e0",
        "usb": "\uf287",
        "usps": "\uf7e1",
        "ussunnah": "\uf407",
        "vaadin": "\uf408",
        "viacoin": "\uf237",
        "viadeo": "\uf2a9",
        "viadeo-square": "\uf2aa",
        "viber": "\uf409",
        "vimeo": "\uf40a",
        "vimeo-square": "\uf194",
        "vimeo-v": "\uf27d",
        "vine": "\uf1ca",
        "vk": "\uf189",
        "vnv": "\uf40b",
        "vuejs": "\uf41f",
        "watchman-monitoring": "\ue087",
        "waze": "\uf83f",
        "weebly": "\uf5cc",
        "weibo": "\uf18a",
        "weixin": "\uf1d7",
        "whatsapp": "\uf232",
        "whatsapp-square": "\uf40c",
        "whmcs": "\uf40d",
        "wikipedia-w": "\uf266",
        "windows": "\uf17a",
        "wirsindhandwerk": "\ue2d0",
        "wsh": "\ue2d0",
        "wix": "\uf5cf",
        "wizards-of-the-coast": "\uf730",
        "wodu": "\ue088",
        "wolf-pack-battalion": "\uf514",
        "wordpress": "\uf19a",
        "wordpress-simple": "\uf411",
        "wpbeginner": "\uf297",
        "wpexplorer": "\uf2de",
        "wpforms": "\uf298",
        "wpressr": "\uf3e4",
        "xbox": "\uf412",
        "xing": "\uf168",
        "xing-square": "\uf169",
        "y-combinator": "\uf23b",
        "yahoo": "\uf19e",
        "yammer": "\uf840",
        "yandex": "\uf413",
        "yandex-international": "\uf414",
        "yarn": "\uf7e3",
        "yelp": "\uf1e9",
        "yoast": "\uf2b1",
        "youtube": "\uf167",
        "youtube-square": "\uf431",
        "zhihu": "\uf63f"
    },
    "regular": {
        "address-book": "\uf2b9",
        "contact-book": "\uf2b9",
        "address-card": "\uf2bb",
        "contact-card": "\uf2bb",
        "vcard": "\uf2bb",
        "bell": "\uf0f3",
        "bell-slash": "\uf1f6",
        "bookmark": "\uf02e",
        "building": "\uf1ad",
        "calendar": "\uf133",
        "calendar-check": "\uf274",
        "calendar-days": "\uf073",
        "calendar-alt": "\uf073",
        "calendar-minus": "\uf272",
        "calendar-plus": "\uf271",
        "calendar-xmark": "\uf273",
        "calendar-times": "\uf273",
        "chart-bar": "\uf080",
        "bar-chart": "\uf080",
        "chess-bishop": "\uf43a",
        "chess-king": "\uf43f",
        "chess-knight": "\uf441",
        "chess-pawn": "\uf443",
        "chess-queen": "\uf445",
        "chess-rook": "\uf447",
        "circle": "\uf111",
        "circle-check": "\uf058",
        "check-circle": "\uf058",
        "circle-dot": "\uf192",
        "dot-circle": "\uf192",
        "circle-down": "\uf358",
        "arrow-alt-circle-down": "\uf358",
        "circle-left": "\uf359",
        "arrow-alt-circle-left": "\uf359",
        "circle-pause": "\uf28b",
        "pause-circle": "\uf28b",
        "circle-play": "\uf144",
        "play-circle": "\uf144",
        "circle-question": "\uf059",
        "question-circle": "\uf059",
        "circle-right": "\uf35a",
        "arrow-alt-circle-right": "\uf35a",
        "circle-stop": "\uf28d",
        "stop-circle": "\uf28d",
        "circle-up": "\uf35b",
        "arrow-alt-circle-up": "\uf35b",
        "circle-user": "\uf2bd",
        "user-circle": "\uf2bd",
        "circle-xmark": "\uf057",
        "times-circle": "\uf057",
        "xmark-circle": "\uf057",
        "clipboard": "\uf328",
        "clock": "\uf017",
        "clock-four": "\uf017",
        "clone": "\uf24d",
        "closed-captioning": "\uf20a",
        "comment": "\uf075",
        "comment-dots": "\uf4ad",
        "commenting": "\uf4ad",
        "comments": "\uf086",
        "compass": "\uf14e",
        "copy": "\uf0c5",
        "copyright": "\uf1f9",
        "credit-card": "\uf09d",
        "credit-card-alt": "\uf09d",
        "envelope": "\uf0e0",
        "envelope-open": "\uf2b6",
        "eye": "\uf06e",
        "eye-slash": "\uf070",
        "face-angry": "\uf556",
        "angry": "\uf556",
        "face-dizzy": "\uf567",
        "dizzy": "\uf567",
        "face-flushed": "\uf579",
        "flushed": "\uf579",
        "face-frown": "\uf119",
        "frown": "\uf119",
        "face-frown-open": "\uf57a",
        "frown-open": "\uf57a",
        "face-grimace": "\uf57f",
        "grimace": "\uf57f",
        "face-grin": "\uf580",
        "grin": "\uf580",
        "face-grin-beam": "\uf582",
        "grin-beam": "\uf582",
        "face-grin-beam-sweat": "\uf583",
        "grin-beam-sweat": "\uf583",
        "face-grin-hearts": "\uf584",
        "grin-hearts": "\uf584",
        "face-grin-squint": "\uf585",
        "grin-squint": "\uf585",
        "face-grin-squint-tears": "\uf586",
        "grin-squint-tears": "\uf586",
        "face-grin-stars": "\uf587",
        "grin-stars": "\uf587",
        "face-grin-tears": "\uf588",
        "grin-tears": "\uf588",
        "face-grin-tongue": "\uf589",
        "grin-tongue": "\uf589",
        "face-grin-tongue-squint": "\uf58a",
        "grin-tongue-squint": "\uf58a",
        "face-grin-tongue-wink": "\uf58b",
        "grin-tongue-wink": "\uf58b",
        "face-grin-wide": "\uf581",
        "grin-alt": "\uf581",
        "face-grin-wink": "\uf58c",
        "grin-wink": "\uf58c",
        "face-kiss": "\uf596",
        "kiss": "\uf596",
        "face-kiss-beam": "\uf597",
        "kiss-beam": "\uf597",
        "face-kiss-wink-heart": "\uf598",
        "kiss-wink-heart": "\uf598",
        "face-laugh": "\uf599",
        "laugh": "\uf599",
        "face-laugh-beam": "\uf59a",
        "laugh-beam": "\uf59a",
        "face-laugh-squint": "\uf59b",
        "laugh-squint": "\uf59b",
        "face-laugh-wink": "\uf59c",
        "laugh-wink": "\uf59c",
        "face-meh": "\uf11a",
        "meh": "\uf11a",
        "face-meh-blank": "\uf5a4",
        "meh-blank": "\uf5a4",
        "face-rolling-eyes": "\uf5a5",
        "meh-rolling-eyes": "\uf5a5",
        "face-sad-cry": "\uf5b3",
        "sad-cry": "\uf5b3",
        "face-sad-tear": "\uf5b4",
        "sad-tear": "\uf5b4",
        "face-smile": "\uf118",
        "smile": "\uf118",
        "face-smile-beam": "\uf5b8",
        "smile-beam": "\uf5b8",
        "face-smile-wink": "\uf4da",
        "smile-wink": "\uf4da",
        "face-surprise": "\uf5c2",
        "surprise": "\uf5c2",
        "face-tired": "\uf5c8",
        "tired": "\uf5c8",
        "file": "\uf15b",
        "file-audio": "\uf1c7",
        "file-code": "\uf1c9",
        "file-excel": "\uf1c3",
        "file-image": "\uf1c5",
        "file-lines": "\uf15c",
        "file-alt": "\uf15c",
        "file-text": "\uf15c",
        "file-pdf": "\uf1c1",
        "file-powerpoint": "\uf1c4",
        "file-video": "\uf1c8",
        "file-word": "\uf1c2",
        "file-zipper": "\uf1c6",
        "file-archive": "\uf1c6",
        "flag": "\uf024",
        "floppy-disk": "\uf0c7",
        "save": "\uf0c7",
        "folder": "\uf07b",
        "folder-blank": "\uf07b",
        "folder-closed": "\ue185",
        "folder-open": "\uf07c",
        "font-awesome": "\uf2b4",
        "font-awesome-flag": "\uf2b4",
        "font-awesome-logo-full": "\uf2b4",
        "futbol": "\uf1e3",
        "futbol-ball": "\uf1e3",
        "soccer-ball": "\uf1e3",
        "gem": "\uf3a5",
        "hand": "\uf256",
        "hand-paper": "\uf256",
        "hand-back-fist": "\uf255",
        "hand-rock": "\uf255",
        "hand-lizard": "\uf258",
        "hand-peace": "\uf25b",
        "hand-point-down": "\uf0a7",
        "hand-point-left": "\uf0a5",
        "hand-point-right": "\uf0a4",
        "hand-point-up": "\uf0a6",
        "hand-pointer": "\uf25a",
        "hand-scissors": "\uf257",
        "hand-spock": "\uf259",
        "handshake": "\uf2b5",
        "hard-drive": "\uf0a0",
        "hdd": "\uf0a0",
        "heart": "\uf004",
        "hospital": "\uf0f8",
        "hospital-alt": "\uf0f8",
        "hospital-wide": "\uf0f8",
        "hourglass": "\uf254",
        "hourglass-2": "\uf254",
        "hourglass-half": "\uf254",
        "id-badge": "\uf2c1",
        "id-card": "\uf2c2",
        "drivers-license": "\uf2c2",
        "image": "\uf03e",
        "images": "\uf302",
        "keyboard": "\uf11c",
        "lemon": "\uf094",
        "life-ring": "\uf1cd",
        "lightbulb": "\uf0eb",
        "map": "\uf279",
        "message": "\uf27a",
        "comment-alt": "\uf27a",
        "money-bill-1": "\uf3d1",
        "money-bill-alt": "\uf3d1",
        "moon": "\uf186",
        "newspaper": "\uf1ea",
        "note-sticky": "\uf249",
        "sticky-note": "\uf249",
        "object-group": "\uf247",
        "object-ungroup": "\uf248",
        "paper-plane": "\uf1d8",
        "paste": "\uf0ea",
        "file-clipboard": "\uf0ea",
        "pen-to-square": "\uf044",
        "edit": "\uf044",
        "rectangle-list": "\uf022",
        "list-alt": "\uf022",
        "rectangle-xmark": "\uf410",
        "rectangle-times": "\uf410",
        "times-rectangle": "\uf410",
        "window-close": "\uf410",
        "registered": "\uf25d",
        "share-from-square": "\uf14d",
        "share-square": "\uf14d",
        "snowflake": "\uf2dc",
        "square": "\uf0c8",
        "square-caret-down": "\uf150",
        "caret-square-down": "\uf150",
        "square-caret-left": "\uf191",
        "caret-square-left": "\uf191",
        "square-caret-right": "\uf152",
        "caret-square-right": "\uf152",
        "square-caret-up": "\uf151",
        "caret-square-up": "\uf151",
        "square-check": "\uf14a",
        "check-square": "\uf14a",
        "square-full": "\uf45c",
        "square-minus": "\uf146",
        "minus-square": "\uf146",
        "square-plus": "\uf0fe",
        "plus-square": "\uf0fe",
        "star": "\uf005",
        "star-half": "\uf089",
        "star-half-stroke": "\uf5c0",
        "star-half-alt": "\uf5c0",
        "sun": "\uf185",
        "thumbs-down": "\uf165",
        "thumbs-up": "\uf164",
        "trash-can": "\uf2ed",
        "trash-alt": "\uf2ed",
        "user": "\uf007",
        "window-maximize": "\uf2d0",
        "window-minimize": "\uf2d1",
        "window-restore": "\uf2d2"
    }
}



from osgeo import osr
import cartopy.crs as ccrs


def cartopy_crs(ds):
    """Converts a pyproj.Proj to a cartopy.crs.Projection
    Parameters
    ----------
    proj: pyproj.Proj
        the projection to convert
    Returns
    -------
    a cartopy.crs.Projection object
    """
    proj = ds.GetProjection()
    inproj = osr.SpatialReference()
    inproj.ImportFromWkt(proj)
    srs = inproj.ExportToProj4()

    km_proj = {'lon_0': 'central_longitude',
               'lat_0': 'central_latitude',
               'x_0': 'false_easting',
               'y_0': 'false_northing',
               'k': 'scale_factor',
               'zone': 'zone',
               }
    km_globe = {'a': 'semimajor_axis',
                'b': 'semiminor_axis',
                }
    km_std = {'lat_1': 'lat_1',
              'lat_2': 'lat_2',
              }
    kw_proj = dict()
    kw_globe = dict()
    kw_std = dict()
    for s in srs.split('+'):
        s = s.split('=')
        if len(s) != 2:
            continue
        k = s[0].strip()
        v = s[1].strip()
        try:
            v = float(v)
        except BaseException:
            pass
        if k == 'proj':
            if v == 'tmerc':
                pycl = ccrs.TransverseMercator
            if v == 'lcc':
                pycl = ccrs.LambertConformal
            if v == 'merc':
                pycl = ccrs.Mercator
            if v == 'utm':
                pycl = ccrs.UTM
            if v == 'aea':
                pycl = ccrs.AlbersEqualArea
        if k in km_proj:
            kw_proj[km_proj[k]] = v
        if k in km_globe:
            kw_globe[km_globe[k]] = v
        if k in km_std:
            kw_std[km_std[k]] = v

    globe = None
    if kw_globe:
        globe = ccrs.Globe(**kw_globe)
    if kw_std:
        kw_proj['standard_parallels'] = (kw_std['lat_1'], kw_std['lat_2'])

    # mercatoooor
    if pycl.__name__ == 'Mercator':
        kw_proj.pop('false_easting', None)
        kw_proj.pop('false_northing', None)

    return pycl(globe=globe, **kw_proj)
